from pydicom.pixels.decoders.base import DeflatedExplicitVRLittleEndianDecoder as DeflatedExplicitVRLittleEndianDecoder, ExplicitVRBigEndianDecoder as ExplicitVRBigEndianDecoder, ExplicitVRLittleEndianDecoder as ExplicitVRLittleEndianDecoder, HTJ2KDecoder as HTJ2KDecoder, HTJ2KLosslessDecoder as HTJ2KLosslessDecoder, HTJ2KLosslessRPCLDecoder as HTJ2KLosslessRPCLDecoder, ImplicitVRLittleEndianDecoder as ImplicitVRLittleEndianDecoder, JPEG2000Decoder as JPEG2000Decoder, JPEG2000LosslessDecoder as JPEG2000LosslessDecoder, JPEGBaseline8BitDecoder as JPEGBaseline8BitDecoder, JPEGExtended12BitDecoder as JPEGExtended12BitDecoder, JPEGLSLosslessDecoder as JPEGLSLosslessDecoder, JPEGLSNearLosslessDecoder as JPEGLSNearLosslessDecoder, JPEGLosslessDecoder as JPEGLosslessDecoder, JPEGLosslessSV1Decoder as JPEGLosslessSV1Decoder, RLELosslessDecoder as RLELosslessDecoder



import argparse
from collections.abc import Callable as Callable
from pydicom.cli.main import filespec_help as filespec_help, filespec_parser as filespec_parser
from pydicom.dataset import Dataset as Dataset

def add_subparser(subparsers: argparse._SubParsersAction) -> None: ...
def do_command(args: argparse.Namespace) -> None: ...
def SOPClassname(ds: Dataset) -> str | None: ...
def quiet_rtplan(ds: Dataset) -> str | None: ...
def quiet_image(ds: Dataset) -> str | None: ...

quiet_items: list[Callable[[Dataset], str | None] | str]

def show_quiet(ds: Dataset) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable as Callable
from types import TracebackType
from typing import Any, Protocol, TypeVar

ExitException = tuple[type[BaseException] | None, BaseException | None, TracebackType | None]
Self = TypeVar('Self', bound='DicomIO')

class ReadableBuffer(Protocol):
    def read(self, size: int = ..., /) -> bytes: ...
    def seek(self, offset: int, whence: int = ..., /) -> int: ...
    def tell(self) -> int: ...

class WriteableBuffer(Protocol):
    def seek(self, offset: int, whence: int = ..., /) -> int: ...
    def tell(self) -> int: ...
    def write(self, b: bytes | bytearray | memoryview, /) -> int: ...

class DicomIO:
    def __init__(self, buffer: ReadableBuffer | WriteableBuffer) -> None: ...
    def close(self, *args: Any, **kwargs: Any) -> Any: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: ExitException) -> None: ...
    @property
    def is_little_endian(self) -> bool: ...
    @is_little_endian.setter
    def is_little_endian(self, value: bool) -> None: ...
    @property
    def is_implicit_VR(self) -> bool: ...
    @is_implicit_VR.setter
    def is_implicit_VR(self, value: bool) -> None: ...
    @property
    def name(self) -> str | None: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def parent(self) -> ReadableBuffer | WriteableBuffer: ...
    def read(self, size: int = -1, /) -> bytes: ...
    def read_exact(self, length: int, nr_retries: int = 3) -> bytes: ...
    def read_tag(self) -> tuple[int, int]: ...
    def read_UL(self) -> int: ...
    def read_US(self) -> int: ...
    def seek(self, offset: int, whence: int = ..., /) -> int: ...
    def tell(self) -> int: ...
    def write(self, b: bytes | bytearray | memoryview, /) -> int: ...
    def write_tag(self, tag: int) -> None: ...
    def write_UL(self, val: int) -> None: ...
    def write_US(self, val: int) -> None: ...

class DicomFileLike(DicomIO): ...

def DicomFile(*args: Any, **kwargs: Any) -> DicomFileLike: ...

class DicomBytesIO(DicomIO):
    getvalue: Incomplete
    def __init__(self, initial_bytes: bytes | bytearray | memoryview = b'') -> None: ...



import argparse
from _typeshed import Incomplete
from collections import deque
from collections.abc import Callable as Callable
from pydicom.cli.main import filespec_help as filespec_help, filespec_parser as filespec_parser
from pydicom.datadict import dictionary_keyword as dictionary_keyword
from pydicom.dataelem import DataElement as DataElement
from pydicom.dataset import Dataset as Dataset
from pydicom.tag import BaseTag as BaseTag
from pydicom.valuerep import AMBIGUOUS_VR as AMBIGUOUS_VR, BYTES_VR as BYTES_VR, VR as VR

line_term: str
first_cap_re: Incomplete
all_cap_re: Incomplete

def camel_to_underscore(name: str) -> str: ...
def tag_repr(tag: BaseTag) -> str: ...
def default_name_filter(name: str) -> str: ...
def code_imports() -> str: ...
def code_dataelem(dataelem: DataElement, dataset_name: str = 'ds', exclude_size: int | None = None, include_private: bool = False, var_names: deque | None = None) -> str: ...
def code_sequence(dataelem: DataElement, dataset_name: str = 'ds', exclude_size: int | None = None, include_private: bool = False, name_filter: Callable[[str], str] = ..., var_names: deque | None = None) -> str: ...
def code_dataset(ds: Dataset, dataset_name: str = 'ds', exclude_size: int | None = None, include_private: bool = False, is_file_meta: bool = False, var_names: deque | None = None) -> str: ...
def code_file(filename: str, exclude_size: int | None = None, include_private: bool = False) -> str: ...
def code_file_from_dataset(ds: Dataset, exclude_size: int | None = None, include_private: bool = False) -> str: ...
def set_parser_arguments(parser: argparse.ArgumentParser, default_exclude_size: int) -> None: ...
def do_codify(args: argparse.Namespace) -> None: ...
def main(default_exclude_size: int, args: list[str] | None = None) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from pydicom.dataset import Dataset as Dataset
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.valuerep import FLOAT_VR as FLOAT_VR, INT_VR as INT_VR, VR as VR
from typing import Any, TypeAlias

JSON_VALUE_KEYS: Incomplete

def convert_to_python_number(value: Any, vr: str) -> Any: ...
OtherValueType = None | str | int | float
PNValueType = None | str | dict[str, str]
SQValueType = dict[str, Any] | None
ValueType: TypeAlias
InlineBinaryType: TypeAlias
BulkDataURIType: TypeAlias
JSONValueType = list[ValueType] | InlineBinaryType | BulkDataURIType
BulkDataType = None | str | int | float | bytes
BulkDataHandlerType = Callable[[str, str, str], BulkDataType] | None

class JsonDataElementConverter:
    dataset_class: Incomplete
    tag: Incomplete
    vr: Incomplete
    value: Incomplete
    value_key: Incomplete
    bulk_data_element_handler: Incomplete
    def __init__(self, dataset_class: type['Dataset'], tag: str, vr: str, value: JSONValueType, value_key: str | None, bulk_data_uri_handler: BulkDataHandlerType | Callable[[str], BulkDataType] | None = None) -> None: ...
    def get_element_values(self) -> Any: ...
    def get_regular_element_value(self, value: ValueType) -> Any: ...
    def get_sequence_item(self, value: SQValueType) -> Dataset: ...
    def get_pn_element_value(self, value: str | dict[str, str]) -> str: ...



from _typeshed import Incomplete
from collections.abc import KeysView
from pydicom.sr._cid_dict import name_for_cid as name_for_cid
from pydicom.sr.coding import Code as Code
from typing import Any

cid_for_name: Incomplete
CIDValueType = dict[str, tuple[str, list[int]]]
ConceptsType = dict[str, CIDValueType]
SnomedMappingType = dict[str, dict[str, str]]

class Collection:
    repr_format: str
    def __init__(self, name: str) -> None: ...
    @property
    def concepts(self) -> dict[str, Code]: ...
    def __contains__(self, item: str | Code) -> bool: ...
    def __dir__(self) -> list[str]: ...
    def dir(self, *filters: str) -> list[str]: ...
    def __getattr__(self, name: str) -> Code: ...
    @property
    def is_cid(self) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def scheme_designator(self) -> str: ...
    def trait_names(self) -> list[str]: ...

class Concepts:
    def __init__(self, collections: list[Collection]) -> None: ...
    @property
    def collections(self) -> KeysView[str]: ...
    def __getattr__(self, name: str) -> Any: ...
    def schemes(self) -> list[str]: ...
    def CIDs(self) -> list[str]: ...

codes: Incomplete



from _typeshed import Incomplete
from collections.abc import Generator
from pydicom.dataelem import RawDataElement as RawDataElement
from typing import Any, Protocol

have_numpy: bool

class ElementCallback(Protocol):
    def __call__(self, raw_elem: RawDataElement, **kwargs: Any) -> RawDataElement: ...

use_DS_decimal: bool
data_element_callback: ElementCallback | None
data_element_callback_kwargs: dict[str, Any]

def reset_data_element_callback() -> None: ...
def DS_numpy(use_numpy: bool = True) -> None: ...
def DS_decimal(use_Decimal_boolean: bool = True) -> None: ...

use_DS_numpy: bool
use_IS_numpy: bool
allow_DS_float: bool
enforce_valid_values: bool
IGNORE: int
WARN: int
RAISE: int

class Settings:
    def __init__(self) -> None: ...
    @property
    def buffered_read_size(self) -> int: ...
    @buffered_read_size.setter
    def buffered_read_size(self, size: int) -> None: ...
    @property
    def reading_validation_mode(self) -> int: ...
    @reading_validation_mode.setter
    def reading_validation_mode(self, value: int) -> None: ...
    @property
    def writing_validation_mode(self) -> int: ...
    @writing_validation_mode.setter
    def writing_validation_mode(self, value: int) -> None: ...
    @property
    def infer_sq_for_un_vr(self) -> bool: ...
    @infer_sq_for_un_vr.setter
    def infer_sq_for_un_vr(self, value: bool) -> None: ...

settings: Incomplete

def disable_value_validation() -> Generator: ...
def strict_reading() -> Generator: ...

convert_wrong_length_to_UN: bool
datetime_conversion: bool
use_none_as_empty_text_VR_value: bool
replace_un_with_known_vr: bool
show_file_meta: bool
logger: Incomplete
pixel_data_handlers: Incomplete
APPLY_J2K_CORRECTIONS: bool
assume_implicit_vr_switch: bool
INVALID_KEYWORD_BEHAVIOR: str
INVALID_KEY_BEHAVIOR: str
debugging: bool

def debug(debug_on: bool = True, default_handler: bool = True) -> None: ...
def future_behavior(enable_future: bool = True) -> None: ...



import PIL
import numpy as np
from pydicom.data import get_palette_files as get_palette_files
from pydicom.dataset import Dataset as Dataset
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.uid import UID as UID
from pydicom.valuerep import VR as VR

HAVE_NP: bool
HAVE_PIL: bool

def apply_color_lut(arr: np.ndarray, ds: Dataset | None = None, palette: str | UID | None = None) -> np.ndarray: ...
def apply_icc_profile(arr: np.ndarray, ds: Dataset | None = None, transform: PIL.ImageCms.ImageCmsTransform | None = None, intent: int | None = None, color_space: str | None = None) -> np.ndarray: ...
def apply_modality_lut(arr: np.ndarray, ds: Dataset) -> np.ndarray: ...
def apply_presentation_lut(arr: np.ndarray, ds: Dataset) -> np.ndarray: ...
apply_rescale = apply_modality_lut

def apply_voi_lut(arr: np.ndarray, ds: Dataset, index: int = 0, prefer_lut: bool = True) -> np.ndarray: ...
def apply_voi(arr: np.ndarray, ds: Dataset, index: int = 0) -> np.ndarray: ...
def apply_windowing(arr: np.ndarray, ds: Dataset, index: int = 0) -> np.ndarray: ...
def convert_color_space(arr: np.ndarray, current: str, desired: str, per_frame: bool = False) -> np.ndarray: ...
def create_icc_transform(ds: Dataset | None = None, icc_profile: bytes = b'', intent: int | None = None, color_space: str | None = None) -> PIL.ImageCms.ImageCmsTransform: ...



from pydicom import config as config, datadict as datadict, values as values
from pydicom.dataelem import RawDataElement as RawDataElement
from pydicom.valuerep import VR as VR
from typing import Any

def fix_separator_callback(raw_elem: RawDataElement, **kwargs: Any) -> RawDataElement: ...
def fix_separator(invalid_separator: bytes, for_VRs: tuple[str, ...] = ('DS', 'IS'), process_unknown_VRs: bool = True) -> None: ...
def fix_mismatch_callback(raw_elem: RawDataElement, **kwargs: Any) -> RawDataElement: ...
def fix_mismatch(with_VRs: tuple[str, ...] = ...) -> None: ...



from ._version import __concepts_version__ as __concepts_version__, __dicom_version__ as __dicom_version__, __version__ as __version__, __version_info__ as __version_info__
from pydicom.dataelem import DataElement as DataElement
from pydicom.dataset import Dataset as Dataset, FileDataset as FileDataset, FileMetaDataset as FileMetaDataset
from pydicom.filereader import dcmread as dcmread
from pydicom.filewriter import dcmwrite as dcmwrite
from pydicom.pixels.utils import iter_pixels as iter_pixels, pixel_array as pixel_array
from pydicom.sequence import Sequence as Sequence

__all__ = ['DataElement', 'Dataset', 'FileDataset', 'FileMetaDataset', 'Sequence', 'dcmread', 'dcmwrite', 'pixel_array', 'iter_pixels', '__version__', '__version_info__', '__dicom_version__', '__concepts_version__']



import numpy as np
from _typeshed import Incomplete
from collections.abc import ByteString as ByteString, Iterable, Iterator
from os import PathLike
from pydicom._dicom_dict import DicomDictionary as DicomDictionary
from pydicom.charset import default_encoding as default_encoding
from pydicom.dataset import Dataset as Dataset
from pydicom.encaps import encapsulate as encapsulate, encapsulate_extended as encapsulate_extended
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.tag import BaseTag as BaseTag
from pydicom.uid import ExplicitVRLittleEndian as ExplicitVRLittleEndian, JPEG2000 as JPEG2000, JPEGLSNearLossless as JPEGLSNearLossless, UID as UID, generate_uid as generate_uid
from pydicom.valuerep import VR as VR
from typing import Any, BinaryIO

HAVE_NP: bool
LOGGER: Incomplete

def as_pixel_options(ds: Dataset, **kwargs: Any) -> dict[str, Any]: ...
def compress(ds: Dataset, transfer_syntax_uid: str, arr: np.ndarray | None = None, *, encoding_plugin: str = '', encapsulate_ext: bool = False, generate_instance_uid: bool = True, jls_error: int | None = None, j2k_cr: list[float] | None = None, j2k_psnr: list[float] | None = None, **kwargs: Any) -> Dataset: ...
def decompress(ds: Dataset, *, as_rgb: bool = True, generate_instance_uid: bool = True, decoding_plugin: str = '', **kwargs: Any) -> Dataset: ...
def expand_ybr422(src: ByteString, bits_allocated: int) -> bytes: ...
def get_expected_length(ds: Dataset, unit: str = 'bytes') -> int: ...
def get_image_pixel_ids(ds: Dataset) -> dict[str, int]: ...
def get_j2k_parameters(codestream: bytes) -> dict[str, Any]: ...
def get_nr_frames(ds: Dataset, warn: bool = True) -> int: ...
def iter_pixels(src: str | PathLike[str] | BinaryIO | Dataset, *, ds_out: Dataset | None = None, specific_tags: list[BaseTag | int] | None = None, indices: Iterable[int] | None = None, raw: bool = False, decoding_plugin: str = '', **kwargs: Any) -> Iterator['np.ndarray']: ...
def pack_bits(arr: np.ndarray, pad: bool = True) -> bytes: ...
def pixel_array(src: str | PathLike[str] | BinaryIO | Dataset, *, ds_out: Dataset | None = None, specific_tags: list[int] | None = None, index: int | None = None, raw: bool = False, decoding_plugin: str = '', **kwargs: Any) -> np.ndarray: ...
def pixel_dtype(ds: Dataset, as_float: bool = False) -> np.dtype: ...
def reshape_pixel_array(ds: Dataset, arr: np.ndarray) -> np.ndarray: ...
def set_pixel_data(ds: Dataset, arr: np.ndarray, photometric_interpretation: str, bits_stored: int, *, generate_instance_uid: bool = True) -> None: ...
def unpack_bits(src: bytes, as_array: bool = True) -> np.ndarray | bytes: ...



import datetime
from _typeshed import Incomplete
from collections.abc import Callable as Callable, Iterator, Sequence
from decimal import Decimal
from enum import Enum
from pydicom import config as config
from pydicom.misc import warn_and_log as warn_and_log
from typing import Any

default_encoding: str
TEXT_VR_DELIMS: Incomplete
PN_DELIMS: Incomplete
MAX_VALUE_LEN: Incomplete
VR_REGEXES: Incomplete
STR_VR_REGEXES: Incomplete
BYTE_VR_REGEXES: Incomplete

def validate_type(vr: str, value: Any, types: type | tuple[type, type]) -> tuple[bool, str]: ...
def validate_vr_length(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_type_and_length(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_regex(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_type_and_regex(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_date_time(vr: str, value: Any, date_time_type: type) -> tuple[bool, str]: ...
def validate_length_and_type_and_regex(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_pn_component_length(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_pn(vr: str, value: Any) -> tuple[bool, str]: ...
def validate_pn_component(value: str | bytes) -> None: ...

VALUE_LENGTH: Incomplete

def validate_number(vr: str, value: Any, min_value: int, max_value: int) -> tuple[bool, str]: ...

VALIDATORS: Incomplete

def validate_value(vr: str, value: Any, validation_mode: int, validator: Callable[[str, Any], tuple[bool, str]] | None = None) -> None: ...

class VR(str, Enum):
    AE = 'AE'
    AS = 'AS'
    AT = 'AT'
    CS = 'CS'
    DA = 'DA'
    DS = 'DS'
    DT = 'DT'
    FD = 'FD'
    FL = 'FL'
    IS = 'IS'
    LO = 'LO'
    LT = 'LT'
    OB = 'OB'
    OD = 'OD'
    OF = 'OF'
    OL = 'OL'
    OW = 'OW'
    OV = 'OV'
    PN = 'PN'
    SH = 'SH'
    SL = 'SL'
    SQ = 'SQ'
    SS = 'SS'
    ST = 'ST'
    SV = 'SV'
    TM = 'TM'
    UC = 'UC'
    UI = 'UI'
    UL = 'UL'
    UN = 'UN'
    UR = 'UR'
    US = 'US'
    UT = 'UT'
    UV = 'UV'
    US_SS_OW = 'US or SS or OW'
    US_SS = 'US or SS'
    US_OW = 'US or OW'
    OB_OW = 'OB or OW'

STANDARD_VR: Incomplete
AMBIGUOUS_VR: Incomplete
DEFAULT_CHARSET_VR: Incomplete
CUSTOMIZABLE_CHARSET_VR: Incomplete
BYTES_VR: Incomplete
FLOAT_VR: Incomplete
INT_VR: Incomplete
LIST_VR: Incomplete
STR_VR = DEFAULT_CHARSET_VR | CUSTOMIZABLE_CHARSET_VR
ALLOW_BACKSLASH: Incomplete
LONG_VALUE_VR: Incomplete
EXPLICIT_VR_LENGTH_16: Incomplete
EXPLICIT_VR_LENGTH_32: Incomplete
BUFFERABLE_VRS: Incomplete

class _DateTimeBase:
    original_string: str
    def __reduce_ex__(self, protocol: int) -> tuple[Any, ...]: ...

class DA(_DateTimeBase, datetime.date):
    def __new__(cls, *args: Any, **kwargs: Any) -> DA | None: ...
    original_string: Incomplete
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class DT(_DateTimeBase, datetime.datetime):
    def __new__(cls, *args: Any, **kwargs: Any) -> DT | None: ...
    original_string: Incomplete
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class TM(_DateTimeBase, datetime.time):
    def __new__(cls, *args: Any, **kwargs: Any) -> TM | None: ...
    original_string: Incomplete
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

def is_valid_ds(s: str) -> bool: ...
def format_number_as_ds(val: float | Decimal) -> str: ...

class DSfloat(float):
    auto_format: bool
    def __new__(cls, val: None | str | int | float | Decimal, auto_format: bool = False, validation_mode: int | None = None) -> str | DSfloat | None: ...
    original_string: Incomplete
    def __init__(self, val: str | int | float | Decimal, auto_format: bool = False, validation_mode: int | None = None) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> int: ...
    def __ne__(self, other: Any) -> Any: ...

class DSdecimal(Decimal):
    auto_format: bool
    def __new__(cls, val: None | str | int | float | Decimal, auto_format: bool = False, validation_mode: int | None = None) -> str | DSdecimal | None: ...
    original_string: Incomplete
    def __init__(self, val: str | int | float | Decimal, auto_format: bool = False, validation_mode: int | None = None) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> int: ...
    def __ne__(self, other: Any) -> Any: ...

DSclass: Any
DSclass = DSdecimal
DSclass = DSfloat

def DS(val: None | str | int | float | Decimal, auto_format: bool = False, validation_mode: int | None = None) -> None | str | DSfloat | DSdecimal: ...

class ISfloat(float):
    def __new__(cls, val: str | float | Decimal, validation_mode: int | None = None) -> float | str: ...
    original_string: Incomplete
    def __init__(self, val: str | float | Decimal, validation_mode: int | None = None) -> None: ...

class IS(int):
    def __new__(cls, val: None | str | int | float | Decimal, validation_mode: int | None = None) -> str | IS | ISfloat | None: ...
    original_string: Incomplete
    def __init__(self, val: str | int | float | Decimal, validation_mode: int | None = None) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> int: ...
    def __ne__(self, other: Any) -> Any: ...

class PersonName:
    def __new__(cls, *args: Any, **kwargs: Any) -> PersonName | None: ...
    original_string: Incomplete
    encodings: Incomplete
    validation_mode: Incomplete
    def __init__(self, val: bytes | str | PersonName, encodings: Sequence[str] | None = None, original_string: bytes | None = None, validation_mode: int | None = None) -> None: ...
    @property
    def components(self) -> tuple[str, ...]: ...
    @property
    def family_name(self) -> str: ...
    @property
    def given_name(self) -> str: ...
    @property
    def middle_name(self) -> str: ...
    @property
    def name_prefix(self) -> str: ...
    @property
    def name_suffix(self) -> str: ...
    @property
    def alphabetic(self) -> str: ...
    @property
    def ideographic(self) -> str: ...
    @property
    def phonetic(self) -> str: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    def __contains__(self, x: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def decode(self, encodings: Sequence[str] | None = None) -> PersonName: ...
    def encode(self, encodings: Sequence[str] | None = None) -> bytes: ...
    def family_comma_given(self) -> str: ...
    def formatted(self, format_str: str) -> str: ...
    def __bool__(self) -> bool: ...
    @classmethod
    def from_named_components(cls, family_name: str | bytes = '', given_name: str | bytes = '', middle_name: str | bytes = '', name_prefix: str | bytes = '', name_suffix: str | bytes = '', family_name_ideographic: str | bytes = '', given_name_ideographic: str | bytes = '', middle_name_ideographic: str | bytes = '', name_prefix_ideographic: str | bytes = '', name_suffix_ideographic: str | bytes = '', family_name_phonetic: str | bytes = '', given_name_phonetic: str | bytes = '', middle_name_phonetic: str | bytes = '', name_prefix_phonetic: str | bytes = '', name_suffix_phonetic: str | bytes = '', encodings: list[str] | None = None) -> PersonName: ...
    @classmethod
    def from_named_components_veterinary(cls, responsible_party_name: str | bytes = '', patient_name: str | bytes = '', responsible_party_name_ideographic: str | bytes = '', patient_name_ideographic: str | bytes = '', responsible_party_name_phonetic: str | bytes = '', patient_name_phonetic: str | bytes = '', encodings: list[str] | None = None) -> PersonName: ...



from _typeshed import Incomplete
from pydicom import config as config
from pydicom._uid_dict import UID_dictionary as UID_dictionary
from pydicom.config import disable_value_validation as disable_value_validation
from pydicom.valuerep import STR_VR_REGEXES as STR_VR_REGEXES, validate_value as validate_value

class UID(str):
    def __new__(cls, val: str, validation_mode: int | None = None) -> UID: ...
    @property
    def is_implicit_VR(self) -> bool: ...
    @property
    def is_little_endian(self) -> bool: ...
    @property
    def is_transfer_syntax(self) -> bool: ...
    @property
    def is_deflated(self) -> bool: ...
    @property
    def is_encapsulated(self) -> bool: ...
    @property
    def is_compressed(self) -> bool: ...
    @property
    def keyword(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def type(self) -> str: ...
    @property
    def info(self) -> str: ...
    @property
    def is_retired(self) -> bool: ...
    @property
    def is_private(self) -> bool: ...
    @property
    def is_valid(self) -> bool: ...
    def set_private_encoding(self, implicit_vr: bool, little_endian: bool) -> None: ...

PYDICOM_ROOT_UID: str
PYDICOM_IMPLEMENTATION_UID: Incomplete
RE_VALID_UID: Incomplete
RE_VALID_UID_PREFIX: Incomplete
ImplicitVRLittleEndian: Incomplete
ExplicitVRLittleEndian: Incomplete
DeflatedExplicitVRLittleEndian: Incomplete
ExplicitVRBigEndian: Incomplete
JPEGBaseline8Bit: Incomplete
JPEGExtended12Bit: Incomplete
JPEGLossless: Incomplete
JPEGLosslessSV1: Incomplete
JPEGLSLossless: Incomplete
JPEGLSNearLossless: Incomplete
JPEG2000Lossless: Incomplete
JPEG2000: Incomplete
JPEG2000MCLossless: Incomplete
JPEG2000MC: Incomplete
MPEG2MPML: Incomplete
MPEG2MPMLF: Incomplete
MPEG2MPHL: Incomplete
MPEG2MPHLF: Incomplete
MPEG4HP41: Incomplete
MPEG4HP41F: Incomplete
MPEG4HP41BD: Incomplete
MPEG4HP41BDF: Incomplete
MPEG4HP422D: Incomplete
MPEG4HP422DF: Incomplete
MPEG4HP423D: Incomplete
MPEG4HP423DF: Incomplete
MPEG4HP42STEREO: Incomplete
MPEG4HP42STEREOF: Incomplete
HEVCMP51: Incomplete
HEVCM10P51: Incomplete
HTJ2KLossless: Incomplete
HTJ2KLosslessRPCL: Incomplete
HTJ2K: Incomplete
JPIPHTJ2KReferenced: Incomplete
JPIPHTJ2KReferencedDeflate: Incomplete
RLELossless: Incomplete
SMPTEST211020UncompressedProgressiveActiveVideo: Incomplete
SMPTEST211020UncompressedInterlacedActiveVideo: Incomplete
SMPTEST211030PCMDigitalAudio: Incomplete
AllTransferSyntaxes: Incomplete
JPEGTransferSyntaxes: Incomplete
JPEGLSTransferSyntaxes: Incomplete
JPEG2000TransferSyntaxes: Incomplete
MPEGTransferSyntaxes: Incomplete
RLETransferSyntaxes: Incomplete
UncompressedTransferSyntaxes: Incomplete
PrivateTransferSyntaxes: Incomplete

def register_transfer_syntax(uid: str | UID, implicit_vr: bool | None = None, little_endian: bool | None = None) -> UID: ...
def generate_uid(prefix: str | None = ..., entropy_srcs: list[str] | None = None) -> UID: ...

MediaStorageDirectoryStorage: Incomplete
ComputedRadiographyImageStorage: Incomplete
DigitalXRayImageStorageForPresentation: Incomplete
DigitalXRayImageStorageForProcessing: Incomplete
DigitalMammographyXRayImageStorageForPresentation: Incomplete
DigitalMammographyXRayImageStorageForProcessing: Incomplete
DigitalIntraOralXRayImageStorageForPresentation: Incomplete
DigitalIntraOralXRayImageStorageForProcessing: Incomplete
EncapsulatedPDFStorage: Incomplete
EncapsulatedCDAStorage: Incomplete
EncapsulatedSTLStorage: Incomplete
EncapsulatedOBJStorage: Incomplete
EncapsulatedMTLStorage: Incomplete
GrayscaleSoftcopyPresentationStateStorage: Incomplete
SegmentedVolumeRenderingVolumetricPresentationStateStorage: Incomplete
MultipleVolumeRenderingVolumetricPresentationStateStorage: Incomplete
VariableModalityLUTSoftcopyPresentationStateStorage: Incomplete
ColorSoftcopyPresentationStateStorage: Incomplete
PseudoColorSoftcopyPresentationStateStorage: Incomplete
BlendingSoftcopyPresentationStateStorage: Incomplete
XAXRFGrayscaleSoftcopyPresentationStateStorage: Incomplete
GrayscalePlanarMPRVolumetricPresentationStateStorage: Incomplete
CompositingPlanarMPRVolumetricPresentationStateStorage: Incomplete
AdvancedBlendingPresentationStateStorage: Incomplete
VolumeRenderingVolumetricPresentationStateStorage: Incomplete
XRayAngiographicImageStorage: Incomplete
EnhancedXAImageStorage: Incomplete
XRayRadiofluoroscopicImageStorage: Incomplete
EnhancedXRFImageStorage: Incomplete
PositronEmissionTomographyImageStorage: Incomplete
LegacyConvertedEnhancedPETImageStorage: Incomplete
XRay3DAngiographicImageStorage: Incomplete
XRay3DCraniofacialImageStorage: Incomplete
BreastTomosynthesisImageStorage: Incomplete
BreastProjectionXRayImageStorageForPresentation: Incomplete
BreastProjectionXRayImageStorageForProcessing: Incomplete
EnhancedPETImageStorage: Incomplete
BasicStructuredDisplayStorage: Incomplete
IntravascularOpticalCoherenceTomographyImageStorageForPresentation: Incomplete
IntravascularOpticalCoherenceTomographyImageStorageForProcessing: Incomplete
CTImageStorage: Incomplete
EnhancedCTImageStorage: Incomplete
LegacyConvertedEnhancedCTImageStorage: Incomplete
NuclearMedicineImageStorage: Incomplete
CTDefinedProcedureProtocolStorage: Incomplete
CTPerformedProcedureProtocolStorage: Incomplete
ProtocolApprovalStorage: Incomplete
XADefinedProcedureProtocolStorage: Incomplete
XAPerformedProcedureProtocolStorage: Incomplete
InventoryStorage: Incomplete
UltrasoundMultiFrameImageStorage: Incomplete
ParametricMapStorage: Incomplete
MRImageStorage: Incomplete
EnhancedMRImageStorage: Incomplete
MRSpectroscopyStorage: Incomplete
EnhancedMRColorImageStorage: Incomplete
LegacyConvertedEnhancedMRImageStorage: Incomplete
RTImageStorage: Incomplete
RTPhysicianIntentStorage: Incomplete
RTSegmentAnnotationStorage: Incomplete
RTRadiationSetStorage: Incomplete
CArmPhotonElectronRadiationStorage: Incomplete
TomotherapeuticRadiationStorage: Incomplete
RoboticArmRadiationStorage: Incomplete
RTRadiationRecordSetStorage: Incomplete
RTRadiationSalvageRecordStorage: Incomplete
TomotherapeuticRadiationRecordStorage: Incomplete
CArmPhotonElectronRadiationRecordStorage: Incomplete
RTDoseStorage: Incomplete
RoboticRadiationRecordStorage: Incomplete
RTRadiationSetDeliveryInstructionStorage: Incomplete
RTTreatmentPreparationStorage: Incomplete
EnhancedRTImageStorage: Incomplete
EnhancedContinuousRTImageStorage: Incomplete
RTPatientPositionAcquisitionInstructionStorage: Incomplete
RTStructureSetStorage: Incomplete
RTBeamsTreatmentRecordStorage: Incomplete
RTPlanStorage: Incomplete
RTBrachyTreatmentRecordStorage: Incomplete
RTTreatmentSummaryRecordStorage: Incomplete
RTIonPlanStorage: Incomplete
RTIonBeamsTreatmentRecordStorage: Incomplete
DICOSCTImageStorage: Incomplete
DICOSDigitalXRayImageStorageForPresentation: Incomplete
DICOSDigitalXRayImageStorageForProcessing: Incomplete
DICOSThreatDetectionReportStorage: Incomplete
DICOS2DAITStorage: Incomplete
DICOS3DAITStorage: Incomplete
DICOSQuadrupoleResonanceStorage: Incomplete
UltrasoundImageStorage: Incomplete
EnhancedUSVolumeStorage: Incomplete
PhotoacousticImageStorage: Incomplete
EddyCurrentImageStorage: Incomplete
EddyCurrentMultiFrameImageStorage: Incomplete
RawDataStorage: Incomplete
SpatialRegistrationStorage: Incomplete
SpatialFiducialsStorage: Incomplete
DeformableSpatialRegistrationStorage: Incomplete
SegmentationStorage: Incomplete
SurfaceSegmentationStorage: Incomplete
TractographyResultsStorage: Incomplete
RealWorldValueMappingStorage: Incomplete
SurfaceScanMeshStorage: Incomplete
SurfaceScanPointCloudStorage: Incomplete
SecondaryCaptureImageStorage: Incomplete
MultiFrameSingleBitSecondaryCaptureImageStorage: Incomplete
MultiFrameGrayscaleByteSecondaryCaptureImageStorage: Incomplete
MultiFrameGrayscaleWordSecondaryCaptureImageStorage: Incomplete
MultiFrameTrueColorSecondaryCaptureImageStorage: Incomplete
VLEndoscopicImageStorage: Incomplete
VideoEndoscopicImageStorage: Incomplete
VLMicroscopicImageStorage: Incomplete
VideoMicroscopicImageStorage: Incomplete
VLSlideCoordinatesMicroscopicImageStorage: Incomplete
VLPhotographicImageStorage: Incomplete
VideoPhotographicImageStorage: Incomplete
OphthalmicPhotography8BitImageStorage: Incomplete
OphthalmicPhotography16BitImageStorage: Incomplete
StereometricRelationshipStorage: Incomplete
OphthalmicTomographyImageStorage: Incomplete
WideFieldOphthalmicPhotographyStereographicProjectionImageStorage: Incomplete
WideFieldOphthalmicPhotography3DCoordinatesImageStorage: Incomplete
OphthalmicOpticalCoherenceTomographyEnFaceImageStorage: Incomplete
OphthalmicOpticalCoherenceTomographyBscanVolumeAnalysisStorage: Incomplete
VLWholeSlideMicroscopyImageStorage: Incomplete
DermoscopicPhotographyImageStorage: Incomplete
ConfocalMicroscopyImageStorage: Incomplete
ConfocalMicroscopyTiledPyramidalImageStorage: Incomplete
LensometryMeasurementsStorage: Incomplete
AutorefractionMeasurementsStorage: Incomplete
KeratometryMeasurementsStorage: Incomplete
SubjectiveRefractionMeasurementsStorage: Incomplete
VisualAcuityMeasurementsStorage: Incomplete
SpectaclePrescriptionReportStorage: Incomplete
OphthalmicAxialMeasurementsStorage: Incomplete
IntraocularLensCalculationsStorage: Incomplete
MacularGridThicknessAndVolumeReportStorage: Incomplete
OphthalmicVisualFieldStaticPerimetryMeasurementsStorage: Incomplete
OphthalmicThicknessMapStorage: Incomplete
CornealTopographyMapStorage: Incomplete
BasicTextSRStorage: Incomplete
EnhancedSRStorage: Incomplete
ComprehensiveSRStorage: Incomplete
Comprehensive3DSRStorage: Incomplete
ExtensibleSRStorage: Incomplete
ProcedureLogStorage: Incomplete
MammographyCADSRStorage: Incomplete
KeyObjectSelectionDocumentStorage: Incomplete
ChestCADSRStorage: Incomplete
XRayRadiationDoseSRStorage: Incomplete
RadiopharmaceuticalRadiationDoseSRStorage: Incomplete
ColonCADSRStorage: Incomplete
ImplantationPlanSRStorage: Incomplete
AcquisitionContextSRStorage: Incomplete
SimplifiedAdultEchoSRStorage: Incomplete
PatientRadiationDoseSRStorage: Incomplete
PlannedImagingAgentAdministrationSRStorage: Incomplete
PerformedImagingAgentAdministrationSRStorage: Incomplete
EnhancedXRayRadiationDoseSRStorage: Incomplete
WaveformAnnotationSRStorage: Incomplete
TwelveLeadECGWaveformStorage: Incomplete
GeneralECGWaveformStorage: Incomplete
AmbulatoryECGWaveformStorage: Incomplete
General32bitECGWaveformStorage: Incomplete
HemodynamicWaveformStorage: Incomplete
CardiacElectrophysiologyWaveformStorage: Incomplete
BasicVoiceAudioWaveformStorage: Incomplete
GeneralAudioWaveformStorage: Incomplete
ArterialPulseWaveformStorage: Incomplete
RespiratoryWaveformStorage: Incomplete
MultichannelRespiratoryWaveformStorage: Incomplete
RoutineScalpElectroencephalogramWaveformStorage: Incomplete
ElectromyogramWaveformStorage: Incomplete
ElectrooculogramWaveformStorage: Incomplete
SleepElectroencephalogramWaveformStorage: Incomplete
BodyPositionWaveformStorage: Incomplete
ContentAssessmentResultsStorage: Incomplete
MicroscopyBulkSimpleAnnotationsStorage: Incomplete
RTBrachyApplicationSetupDeliveryInstructionStorage: Incomplete
RTBeamsDeliveryInstructionStorage: Incomplete
HangingProtocolStorage: Incomplete
ColorPaletteStorage: Incomplete
GenericImplantTemplateStorage: Incomplete
ImplantAssemblyTemplateStorage: Incomplete
ImplantTemplateGroupStorage: Incomplete





from pydicom import Dataset

ds = Dataset()
ds.NumberOfFrames = ["foo"]
reveal_type(ds.SamplesPerPixel)
ds.SamplesPerPixel = "abcd"
reveal_type(ds.PixelData)


"""
If the package is working correctly this should result in the following
when running `mypy tests/` (exit code 1):

tests/test_function.py:6: error: Incompatible types in assignment (expression has type "list[str]", variable has type "str | int | IS | ISfloat | None")  [assignment]
tests/test_function.py:7: note: Revealed type is "builtins.int"
tests/test_function.py:8: error: Incompatible types in assignment (expression has type "str", variable has type "int")  [assignment]
Found 2 errors in 1 file (checked 1 source file)


Otherwise it will result in (exit code 0):

tests/test_function.py:7: note: Revealed type is "Any"
Success: no issues found in 1 source file
"""



from _typeshed import Incomplete
from pathlib import Path

LOGGER: Incomplete

def size_in_bytes(expr: int | float | str | None) -> None | float | int: ...
def is_dicom(file_path: str | Path) -> bool: ...
def warn_and_log(msg: str, category: type[Warning] | None = None, stacklevel: int = 1) -> None: ...



import argparse
from _typeshed import Incomplete
from pydicom import dcmread as dcmread
from pydicom.data.data_manager import get_charset_files as get_charset_files, get_testdata_file as get_testdata_file
from pydicom.dataset import Dataset as Dataset
from typing import Any

subparsers: argparse._SubParsersAction | None
re_kywd_or_item: str
re_file_spec_object: Incomplete
filespec_help: str

def eval_element(ds: Dataset, element: str) -> Any: ...
def filespec_parts(filespec: str) -> tuple[str, str, str]: ...
def filespec_parser(filespec: str) -> list[tuple[Dataset, Any]]: ...
def help_command(args: argparse.Namespace) -> None: ...

SubCommandType: Incomplete

def get_subcommand_entry_points() -> SubCommandType: ...
def main(args: list[str] | None = None) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable as Callable, MutableSequence
from pydicom import config as config
from pydicom.datadict import dictionary_VR as dictionary_VR, private_dictionary_VR as private_dictionary_VR
from pydicom.dataelem import RawDataElement as RawDataElement
from pydicom.dataset import Dataset as Dataset
from pydicom.errors import BytesLengthException as BytesLengthException
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.tag import BaseTag as BaseTag
from pydicom.valuerep import VR as VR
from typing import Any, Protocol

class RawDataHook(Protocol):
    def __call__(self, raw: RawDataElement, data: dict[str, Any], *, encoding: str | MutableSequence[str] | None = ..., ds: Dataset | None = ..., **kwargs: Any) -> None: ...

class Hooks:
    raw_element_value: Incomplete
    raw_element_vr: Incomplete
    raw_element_kwargs: Incomplete
    def __init__(self) -> None: ...
    def register_callback(self, hook: str, func: Callable) -> None: ...
    def register_kwargs(self, hook: str, kwargs: dict[str, Any]) -> None: ...

def raw_element_vr(raw: RawDataElement, data: dict[str, Any], *, encoding: str | MutableSequence[str] | None = None, ds: Dataset | None = None, **kwargs: Any) -> None: ...
def raw_element_value(raw: RawDataElement, data: dict[str, Any], *, encoding: str | MutableSequence[str] | None = None, ds: Dataset | None = None, **kwargs: Any) -> None: ...
def raw_element_value_fix_separator(raw: RawDataElement, data: dict[str, Any], *, encoding: str | MutableSequence[str] | None = None, ds: Dataset | None, separator: str | bytes = b',', target_VRs: tuple[str, ...] | None = None, **kwargs: Any) -> None: ...
def raw_element_value_retry(raw: RawDataElement, data: dict[str, Any], *, encoding: str | MutableSequence[str] | None = None, ds: Dataset | None, target_VRs: dict[str, tuple[str, ...]] | None = None, **kwargs: Any) -> None: ...

hooks: Hooks




from pathlib import Path
from typing import Any

from pydicom._dicom_dict import DicomDictionary

BASE_DIRECTORY = Path(__file__).parent.parent
DS_DST = BASE_DIRECTORY / "custom" / "dataset.pyi"
DS_SRC = BASE_DIRECTORY / "src" / "pydicom-stubs" / "dataset.pyi"


ElementDictType = dict[int, tuple[str, str, str, str]]


def generate_dataset_hints(d: ElementDictType, indent: str = "    ") -> list[str]:
    """Generate element type hints for the Dataset class."""
    exclusions = {
        "FileMetaInformationGroupLength",
        "FileMetaInformationVersion",
        "MediaStorageSOPClassUID",
        "MediaStorageSOPInstanceUID",
        "TransferSyntaxUID",
        "ImplementationClassUID",
        "ImplementationVersionName",
        "SourceApplicationEntityTitle",
        "SendingApplicationEntityTitle",
        "ReceivingApplicationEntityTitle",
        "SourcePresentationAddress",
        "ReceivingPresentationAddress",
        "PrivateInformationCreatorUID",
        "PrivateInformation",
        "SamplesPerPixel",
        "PlanarConfiguration",
        "PhotometricInterpretation",
        "Rows",
        "Columns",
        "BitsAllocated",
        "BitsStored",
        "HighBit",
        "PixelRepresentation",
    }
    custom_hints = [
        "# Keyword: type hint  # VR, VM, Type",
        "# Part 10, 7.1: File Meta Information Elements"
        "FileMetaInformationGroupLength: int  # UL, 1, 1",
        "FileMetaInformationVersion: bytes  # OB, 1, 1",
        "MediaStorageSOPClassUID: str | UID  # UI, 1, 1",
        "MediaStorageSOPInstanceUID: str | UID  # UI, 1, 1",
        "TransferSyntaxUID: str | UID  # UI, 1, 1",
        "ImplementationClassUID: str | UID  # UI, 1, 1",
        "ImplementationVersionName: str | None  # SH, 1, 3",
        "SourceApplicationEntityTitle: str | None  # AE, 1, 3",
        "SendingApplicationEntityTitle: str | None  # AE, 1, 3",
        "ReceivingApplicationEntityTitle: str | None  # AE, 1, 3",
        "SourcePresentationAddress: str | None  # UR, 1, 3",
        "ReceivingPresentationAddress: str | None  # UR, 1, 3",
        "PrivateInformationCreatorUID: str | UID | None  # UI, 1, 3",
        "PrivateInformation: bytes  # OB, 1, 1C",
        "",
        "# Part 3, C.7.6.3.3: Image Pixel Description Macro",
        "SamplesPerPixel: int  # US, 1, 1",
        "PlanarConfiguration: int  # US, 1, 1C",
        "PhotometricInterpretation: str  # CS, 1, 1",
        "Rows: int  # US, 1, 1",
        "Columns: int  # US, 1, 1",
        "BitsAllocated: int  # US, 1, 1",
        "BitsStored: int  # US, 1, 1",
        "HighBit: int  # US, 1, 1",
        "PixelRepresentation: int  # US, 1, 1",
        "",
        "# Part 6, Table 6-1: All other public elements",
    ]

    hints = [f"{indent}{hint}\n" for hint in custom_hints]
    for tag, (vr, vm, name, retired, keyword) in d.items():
        if (
            not keyword
            or keyword in exclusions
            or retired == "Retired"
            or vr == "NONE"
        ):
            continue

        if len(vr) > 2:
            # Ambiguous VRs
            vr = vr.replace(" or ", "_")
            hints.append(f"{indent}{keyword}: {vr}_Type\n")
            continue

        if vr == "SQ":
            hints.append(f"{indent}{keyword}: SQType\n")
            continue

        if vm == "1":
            hints.append(f"{indent}{keyword}: {vr}_1_Type\n")
        elif vm == "1-n":
            hints.append(f"{indent}{keyword}: {vr}_1N_Type\n")
        else:
            hints.append(f"{indent}{keyword}: {vr}_N_Type\n")

    return hints


def write_dataset_stub(d: ElementDictType) -> None:
    """Update the custom/dataset.pyi file."""

    modified = []
    modified.append("from pydicom._type_definitions import *\n")

    with open(DS_SRC, 'r') as f:
        for idx, line in enumerate(f.readlines()):
            modified.append(line)
            if line.startswith("class Dataset:"):
                modified.append("    # Auto-generated class attributes below\n")
                modified.append("\n")
                modified.extend(generate_dataset_hints(d))
                modified.append("\n")
                modified.append("    # Auto-generated class attributes above\n")
                modified.append("\n")

    with open(DS_DST, "w") as f:
        f.write("".join(modified))


if __name__ == "__main__":
    write_dataset_stub(DicomDictionary)



import os
from collections.abc import Generator, Iterator
from io import BufferedIOBase
from pydicom.config import logger as logger, settings as settings
from pydicom.datadict import dictionary_description as dictionary_description
from pydicom.filebase import ReadableBuffer as ReadableBuffer, WriteableBuffer as WriteableBuffer
from pydicom.misc import size_in_bytes as size_in_bytes
from pydicom.tag import BaseTag as BaseTag, ItemTag as ItemTag, SequenceDelimiterTag as SequenceDelimiterTag, Tag as Tag, TupleTag as TupleTag
from typing import BinaryIO

PathType = str | bytes | os.PathLike

def absorb_delimiter_item(fp: BinaryIO, is_little_endian: bool, delimiter: BaseTag) -> None: ...
def find_bytes(fp: BinaryIO, bytes_to_find: bytes, read_size: int = 128, rewind: bool = True) -> int | None: ...
def read_undefined_length_value(fp: BinaryIO, is_little_endian: bool, delimiter_tag: BaseTag, defer_size: int | float | None = None, read_size: int = ...) -> bytes | None: ...
def find_delimiter(fp: BinaryIO, delimiter: BaseTag, is_little_endian: bool, read_size: int = 128, rewind: bool = True) -> int | None: ...
def length_of_undefined_length(fp: BinaryIO, delimiter: BaseTag, is_little_endian: bool, read_size: int = 128, rewind: bool = True) -> int | None: ...
def path_from_pathlike(file_object: PathType | BinaryIO | ReadableBuffer | WriteableBuffer) -> str | BinaryIO: ...
def check_buffer(buffer: BufferedIOBase) -> None: ...
def reset_buffer_position(buffer: BufferedIOBase) -> Generator[int, None, None]: ...
def read_buffer(buffer: BufferedIOBase, *, chunk_size: int | None = None) -> Iterator[bytes]: ...
def buffer_length(buffer: BufferedIOBase) -> int: ...
def buffer_remaining(buffer: BufferedIOBase) -> int: ...
def buffer_equality(buffer: BufferedIOBase, other: bytes | bytearray | BufferedIOBase) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Callable as Callable, Iterable
from pydicom import config as config
from pydicom.charset import convert_encodings as convert_encodings, default_encoding as default_encoding, encode_string as encode_string
from pydicom.dataelem import DataElement as DataElement, RawDataElement as RawDataElement, convert_raw_data_element as convert_raw_data_element
from pydicom.dataset import Dataset as Dataset, FileMetaDataset as FileMetaDataset, validate_file_meta as validate_file_meta
from pydicom.filebase import DicomBytesIO as DicomBytesIO, DicomFile as DicomFile, DicomIO as DicomIO, WriteableBuffer as WriteableBuffer
from pydicom.fileutil import PathType as PathType, buffer_remaining as buffer_remaining, path_from_pathlike as path_from_pathlike, read_buffer as read_buffer, reset_buffer_position as reset_buffer_position
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.multival import MultiValue as MultiValue
from pydicom.tag import BaseTag as BaseTag, ItemDelimiterTag as ItemDelimiterTag, ItemTag as ItemTag, SequenceDelimiterTag as SequenceDelimiterTag, Tag as Tag, tag_in_exception as tag_in_exception
from pydicom.uid import DeflatedExplicitVRLittleEndian as DeflatedExplicitVRLittleEndian, ExplicitVRBigEndian as ExplicitVRBigEndian, ImplicitVRLittleEndian as ImplicitVRLittleEndian, UID as UID
from pydicom.valuerep import AMBIGUOUS_VR as AMBIGUOUS_VR, CUSTOMIZABLE_CHARSET_VR as CUSTOMIZABLE_CHARSET_VR, DA as DA, DSclass as DSclass, DT as DT, EXPLICIT_VR_LENGTH_32 as EXPLICIT_VR_LENGTH_32, IS as IS, PersonName as PersonName, TM as TM, VR as VR
from pydicom.values import convert_numbers as convert_numbers
from typing import Any, BinaryIO

def correct_ambiguous_vr_element(elem: DataElement | RawDataElement, ds: Dataset, is_little_endian: bool, ancestors: list[Dataset] | None = None) -> DataElement | RawDataElement: ...
def correct_ambiguous_vr(ds: Dataset, is_little_endian: bool, ancestors: list[Dataset] | None = None) -> Dataset: ...
def write_numbers(fp: DicomIO, elem: DataElement, struct_format: str) -> None: ...
def write_OBvalue(fp: DicomIO, elem: DataElement) -> None: ...
def write_OWvalue(fp: DicomIO, elem: DataElement) -> None: ...
def write_UI(fp: DicomIO, elem: DataElement) -> None: ...
def multi_string(val: str | Iterable[str]) -> str: ...
def write_PN(fp: DicomIO, elem: DataElement, encodings: list[str] | None = None) -> None: ...
def write_string(fp: DicomIO, elem: DataElement, padding: str = ' ') -> None: ...
def write_text(fp: DicomIO, elem: DataElement, encodings: list[str] | None = None) -> None: ...
def write_number_string(fp: DicomIO, elem: DataElement) -> None: ...
def write_DA(fp: DicomIO, elem: DataElement) -> None: ...
def write_DT(fp: DicomIO, elem: DataElement) -> None: ...
def write_TM(fp: DicomIO, elem: DataElement) -> None: ...
def write_data_element(fp: DicomIO, elem: DataElement | RawDataElement, encodings: str | list[str] | None = None) -> None: ...
EncodingType = tuple[bool | None, bool | None]

def write_dataset(fp: DicomIO, dataset: Dataset, parent_encoding: str | list[str] = ...) -> int: ...
def write_sequence(fp: DicomIO, elem: DataElement, encodings: list[str]) -> None: ...
def write_sequence_item(fp: DicomIO, dataset: Dataset, encodings: list[str]) -> None: ...
def write_UN(fp: DicomIO, elem: DataElement) -> None: ...
def write_ATvalue(fp: DicomIO, elem: DataElement) -> None: ...
def write_file_meta_info(fp: DicomIO, file_meta: FileMetaDataset, enforce_standard: bool = True) -> None: ...
def dcmwrite(filename: PathType | BinaryIO | WriteableBuffer, dataset: Dataset, __write_like_original: bool | None = None, /, *, implicit_vr: bool | None = None, little_endian: bool | None = None, enforce_file_format: bool = False, force_encoding: bool = False, overwrite: bool = True, **kwargs: Any) -> None: ...

writers: Incomplete



from collections.abc import Callable as Callable, Iterable, Iterator, MutableSequence
from typing import Any, TypeVar, overload

T = TypeVar('T')
Self = TypeVar('Self', bound='ConstrainedList')

class ConstrainedList(MutableSequence[T]):
    def __init__(self, iterable: Iterable[T] | None = None) -> None: ...
    def append(self, item: T) -> None: ...
    def __delitem__(self, index: slice | int) -> None: ...
    def extend(self, val: Iterable[T]) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    @overload
    def __getitem__(self, index: int) -> T: ...
    @overload
    def __getitem__(self, index: slice) -> MutableSequence[T]: ...
    def __iadd__(self, other: Iterable[T]) -> Self: ...
    def insert(self, position: int, item: T) -> None: ...
    def __iter__(self) -> Iterator[T]: ...
    def __len__(self) -> int: ...
    def __ne__(self, other: Any) -> Any: ...
    @overload
    def __setitem__(self, idx: int, val: T) -> None: ...
    @overload
    def __setitem__(self, idx: slice, val: Iterable[T]) -> None: ...

class MultiValue(ConstrainedList[T]):
    def __init__(self, type_constructor: Callable[[Any], T], iterable: Iterable[Any]) -> None: ...
    def sort(self, *args: Any, **kwargs: Any) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from typing import Any, TypeAlias

def tag_in_exception(tag: BaseTag) -> Iterator[None]: ...

TagType: TypeAlias
TagListType: Incomplete

def Tag(arg: TagType, arg2: int | None = None) -> BaseTag: ...

class BaseTag(int):
    def __le__(self, other: Any) -> Any: ...
    def __lt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    __hash__: Incomplete
    @property
    def json_key(self) -> str: ...
    @property
    def group(self) -> int: ...
    @property
    def element(self) -> int: ...
    elem = element
    @property
    def is_private(self) -> bool: ...
    @property
    def is_private_creator(self) -> bool: ...
    @property
    def private_creator(self) -> BaseTag: ...

def TupleTag(group_elem: tuple[int, int]) -> BaseTag: ...

ItemTag: Incomplete
ItemDelimiterTag: Incomplete
SequenceDelimiterTag: Incomplete
TAG_PIXREP: Incomplete
TAG_CHARSET: Incomplete



from typing import Any, NamedTuple

class Code(NamedTuple):
    value: str
    scheme_designator: str
    meaning: str
    scheme_version: str | None = ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...



from pydicom.pixels.decoders.base import get_decoder as get_decoder
from pydicom.pixels.encoders.base import get_encoder as get_encoder
from pydicom.pixels.processing import apply_color_lut as apply_color_lut, apply_icc_profile as apply_icc_profile, apply_modality_lut as apply_modality_lut, apply_presentation_lut as apply_presentation_lut, apply_rescale as apply_rescale, apply_voi as apply_voi, apply_voi_lut as apply_voi_lut, apply_windowing as apply_windowing, convert_color_space as convert_color_space, create_icc_transform as create_icc_transform
from pydicom.pixels.utils import as_pixel_options as as_pixel_options, compress as compress, decompress as decompress, iter_pixels as iter_pixels, pack_bits as pack_bits, pixel_array as pixel_array, set_pixel_data as set_pixel_data, unpack_bits as unpack_bits



from collections.abc import Callable as Callable
from enum import Enum
from pydicom.dataset import Dataset as Dataset
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.pixels.decoders.base import DecodeFunction as DecodeFunction, DecodeOptions as DecodeOptions
from pydicom.pixels.encoders.base import EncodeFunction as EncodeFunction, EncodeOptions as EncodeOptions
from pydicom.pixels.utils import as_pixel_options as as_pixel_options
from pydicom.uid import UID as UID
from typing import Any, TypedDict

Buffer = bytes | bytearray | memoryview

class CoderBase:
    def __init__(self, uid: UID, decoder: bool) -> None: ...
    def add_plugin(self, label: str, import_path: tuple[str, str]) -> None: ...
    def add_plugins(self, plugins: list[tuple[str, tuple[str, str]]]) -> None: ...
    @property
    def available_plugins(self) -> tuple[str, ...]: ...
    @property
    def is_available(self) -> bool: ...
    @property
    def is_encapsulated(self) -> bool: ...
    @property
    def is_native(self) -> bool: ...
    @property
    def missing_dependencies(self) -> list[str]: ...
    def remove_plugin(self, label: str) -> None: ...
    @property
    def UID(self) -> UID: ...

class PhotometricInterpretation(str, Enum):
    MONOCHROME1 = 'MONOCHROME1'
    MONOCHROME2 = 'MONOCHROME2'
    PALETTE_COLOR = 'PALETTE COLOR'
    RGB = 'RGB'
    YBR_FULL = 'YBR_FULL'
    YBR_FULL_422 = 'YBR_FULL_422'
    YBR_ICT = 'YBR_ICT'
    YBR_RCT = 'YBR_RCT'
    HSV = 'HSV'
    ARGB = 'ARGB'
    CMYK = 'CMYK'
    YBR_PARTIAL_422 = 'YBR_PARTIAL_422'
    YBR_PARTIAL_420 = 'YBR_PARTIAL_420'

class RunnerBase:
    def __init__(self, tsyntax: UID) -> None: ...
    @property
    def bits_allocated(self) -> int: ...
    @property
    def bits_stored(self) -> int: ...
    @property
    def columns(self) -> int: ...
    def del_option(self, name: str) -> None: ...
    @property
    def extended_offsets(self) -> tuple[list[int], list[int]] | tuple[bytes, bytes] | None: ...
    def frame_length(self, unit: str = 'bytes') -> int: ...
    def get_option(self, name: str, default: Any = None) -> Any: ...
    @property
    def is_array(self) -> bool: ...
    @property
    def is_binary(self) -> bool: ...
    @property
    def is_buffer(self) -> bool: ...
    @property
    def is_dataset(self) -> bool: ...
    @property
    def number_of_frames(self) -> int: ...
    @property
    def options(self) -> DecodeOptions | EncodeOptions: ...
    @property
    def photometric_interpretation(self) -> str: ...
    @property
    def pixel_keyword(self) -> str: ...
    @property
    def pixel_representation(self) -> int: ...
    @property
    def planar_configuration(self) -> int: ...
    @property
    def rows(self) -> int: ...
    @property
    def samples_per_pixel(self) -> int: ...
    def set_option(self, name: str, value: Any) -> None: ...
    def set_options(self, **kwargs: DecodeOptions | EncodeOptions) -> None: ...
    @property
    def transfer_syntax(self) -> UID: ...
    def validate(self) -> None: ...

class RunnerOptions(TypedDict, total=False):
    bits_allocated: int
    bits_stored: int
    columns: int
    number_of_frames: int
    photometric_interpretation: str
    pixel_keyword: str
    pixel_representation: int
    rows: int
    samples_per_pixel: int
    transfer_syntax_uid: UID
    planar_configuration: int
    extended_offsets: tuple[bytes, bytes] | tuple[list[int], list[int]]




import os
from pathlib import Path
import shutil
import subprocess
import sys

from pydicom import __version__


BASE_DIRECTORY = Path(__file__).parent.parent
SRC_DIRECTORY = BASE_DIRECTORY / "custom"
DST_DIRECTORY = BASE_DIRECTORY / "src" / "pydicom-stubs"
PYDICOM_DIRECTORY = BASE_DIRECTORY.parent / "pydicom" / "src" / "pydicom"


REMOVALS = [
    DST_DIRECTORY / "pixels" / "decoders" / "gdcm.pyi",
    DST_DIRECTORY / "pixels" / "decoders" / "pillow.pyi",
    DST_DIRECTORY / "pixels" / "decoders" / "pyjpegls.pyi",
    DST_DIRECTORY / "pixels" / "decoders" / "pylibjpeg.pyi",
    DST_DIRECTORY / "pixels" / "decoders" / "rle.pyi",
    DST_DIRECTORY / "pixels" / "encoders" / "gdcm.pyi",
    DST_DIRECTORY / "pixels" / "encoders" / "native.pyi",
    DST_DIRECTORY / "pixels" / "encoders" / "pyjpegls.pyi",
    DST_DIRECTORY / "pixels" / "encoders" / "pylibjpeg.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "gdcm_handler.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "jpeg_ls_handler.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "numpy_handler.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "pillow_handler.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "pylibjpeg_handler.pyi",
    DST_DIRECTORY / "pixel_data_handlers" / "rle_handler.pyi",
]


def update_stubs() -> None:
    # Clear out the stub files
    if DST_DIRECTORY.exists():
        shutil.rmtree(DST_DIRECTORY)

    if (BASE_DIRECTORY / "pydicom").exists():
        shutil.rmtree(BASE_DIRECTORY / "pydicom")

    print("Generating basic stub files with stubgen")
    p = subprocess.run(
        [f"stubgen {os.fspath(PYDICOM_DIRECTORY)} -o ."],
        shell=True,
    )

    if p.returncode != 0:
        print("  Failed to generate the basic stub files")
        sys.exit(1)

    print(f"Moving basic stub files to {DST_DIRECTORY}")
    shutil.move(BASE_DIRECTORY / "pydicom", DST_DIRECTORY)

    print("Generating custom stub files")
    if not SRC_DIRECTORY.exists():
        SRC_DIRECTORY.mkdir(parents=True, exist_ok=True)

    subprocess.run(["python scripts/generate_stubs.py"], shell=True)

    print("Replacing basic stub files with custom ones")
    for path in SRC_DIRECTORY.glob("*.pyi"):
        shutil.copyfile(path, DST_DIRECTORY / path.name)

    print("Removing unnecessary stubs")
    for path in REMOVALS:
        path.unlink(missing_ok=True)


def update_version() -> None:
    # Get the current package version
    typd_version = ""
    contents = []
    with open(BASE_DIRECTORY / "pyproject.toml", "r") as f:
        for line in f.readlines():
            contents.append(line.rstrip())
            if line.startswith("version = "):
                typd_version = line.rstrip()

    # types-pydicom version: X.Y.Z.N[.dev0]
    typd_version = typd_version.strip("version = ")
    typd_version = typd_version.strip("\"")
    print(f"Found current package version '{typd_version}'")
    typd_version = typd_version.split(".")
    if not typd_version or len(typd_version) < 3:
        raise RuntimeError(
            f"Unable to determine the current package version from '{typd_version}'"
        )

    # pydicom version: X.Y.Z[.dev0]
    pyd_version = __version__.strip().split(".")
    if len(pyd_version) not in (3, 4):
        raise RuntimeError(f"Unexpected pydicom version string '{pyd_version}'")

    # Determine new package version
    if pyd_version[-1] == "dev0":
        # If pydicom is dev0 then use X.Y.Z.0.dev0:
        version = f"{'.'.join(pyd_version[:-1])}.0.dev0"
    elif pyd_version == typd_version[:3]:
        # If X.Y.Z match -> increment N
        version = f"{'.'.join(pyd_version)}.{int(typd_version[3]) + 1}"
    else:
        # If X.Y.Z don't match, use X.Y.Z.0
        version = f"{'.'.join(pyd_version)}.0"

    if version.split(".") == typd_version:
        print(f"No package version change required")
        return

    print(f"Changing package version to '{version}'")
    for idx, line in enumerate(contents):
        if line.startswith("version = "):
            contents[idx] = f"version = \"{version}\""
            break

    with open(BASE_DIRECTORY / "pyproject.toml", "w") as f:
        f.write("\n".join(contents))


if __name__ == "__main__":
    update_stubs()
    update_version()



from _typeshed import Incomplete
from collections.abc import Iterable
from pydicom.dataset import Dataset as Dataset
from pydicom.multival import ConstrainedList as ConstrainedList
from typing import TypeVar

Self = TypeVar('Self', bound='Sequence')

class Sequence(ConstrainedList[Dataset]):
    is_undefined_length: Incomplete
    def __init__(self, iterable: Iterable[Dataset] | None = None) -> None: ...
    def extend(self, val: Iterable[Dataset]) -> None: ...
    def __iadd__(self, other: Iterable[Dataset]) -> Self: ...
    def __setitem__(self, index: slice | int, val: Iterable[Dataset] | Dataset) -> None: ...



from pydicom.pixels.encoders.base import JPEG2000Encoder as JPEG2000Encoder, JPEG2000LosslessEncoder as JPEG2000LosslessEncoder, JPEGLSLosslessEncoder as JPEGLSLosslessEncoder, JPEGLSNearLosslessEncoder as JPEGLSNearLosslessEncoder, RLELosslessEncoder as RLELosslessEncoder



from _typeshed import Incomplete
from pydicom._dicom_dict import DicomDictionary as DicomDictionary, RepeatersDictionary as RepeatersDictionary
from pydicom._private_dict import private_dictionaries as private_dictionaries
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.tag import BaseTag as BaseTag, Tag as Tag, TagType as TagType

masks: dict[str, tuple[int, int]]
mask1: Incomplete
mask2: Incomplete

def mask_match(tag: int) -> str | None: ...
def add_dict_entry(tag: int, VR: str, keyword: str, description: str, VM: str = '1', is_retired: str = '') -> None: ...
def add_dict_entries(new_entries_dict: dict[int, tuple[str, str, str, str, str]]) -> None: ...
def add_private_dict_entry(private_creator: str, tag: int, VR: str, description: str, VM: str = '1') -> None: ...
def add_private_dict_entries(private_creator: str, new_entries_dict: dict[int, tuple[str, str, str, str]]) -> None: ...
def get_entry(tag: TagType) -> tuple[str, str, str, str, str]: ...
def dictionary_is_retired(tag: TagType) -> bool: ...
def dictionary_VR(tag: TagType) -> str: ...
def dictionary_VM(tag: TagType) -> str: ...
def dictionary_description(tag: TagType) -> str: ...
def dictionary_keyword(tag: TagType) -> str: ...
def dictionary_has_tag(tag: TagType) -> bool: ...
def keyword_for_tag(tag: TagType) -> str: ...

keyword_dict: dict[str, int]

def tag_for_keyword(keyword: str) -> int | None: ...
def repeater_has_tag(tag: int) -> bool: ...

REPEATER_KEYWORDS: Incomplete

def repeater_has_keyword(keyword: str) -> bool: ...
def get_private_entry(tag: TagType, private_creator: str) -> tuple[str, str, str, str]: ...
def private_dictionary_VR(tag: TagType, private_creator: str) -> str: ...
def private_dictionary_VM(tag: TagType, private_creator: str) -> str: ...
def private_dictionary_description(tag: TagType, private_creator: str) -> str: ...



from _typeshed import Incomplete
from collections.abc import Callable as Callable, MutableSequence
from pydicom import config as config, jsonrep as jsonrep
from pydicom.config import logger as logger
from pydicom.datadict import dictionary_VR as dictionary_VR, dictionary_description as dictionary_description, dictionary_has_tag as dictionary_has_tag, dictionary_is_retired as dictionary_is_retired, dictionary_keyword as dictionary_keyword, private_dictionary_description as private_dictionary_description, repeater_has_tag as repeater_has_tag
from pydicom.dataset import Dataset as Dataset
from pydicom.fileutil import buffer_equality as buffer_equality, buffer_length as buffer_length, check_buffer as check_buffer
from pydicom.hooks import hooks as hooks
from pydicom.jsonrep import BulkDataType as BulkDataType, JsonDataElementConverter as JsonDataElementConverter
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.multival import MultiValue as MultiValue
from pydicom.tag import BaseTag as BaseTag, Tag as Tag
from pydicom.uid import UID as UID
from pydicom.valuerep import ALLOW_BACKSLASH as ALLOW_BACKSLASH, AMBIGUOUS_VR as AMBIGUOUS_VR, BUFFERABLE_VRS as BUFFERABLE_VRS, BYTES_VR as BYTES_VR, DEFAULT_CHARSET_VR as DEFAULT_CHARSET_VR, LONG_VALUE_VR as LONG_VALUE_VR, PersonName as PersonName, STR_VR as STR_VR, validate_value as validate_value
from typing import Any, NamedTuple

def empty_value_for_VR(VR: str | None, raw: bool = False) -> bytes | list[str] | str | None | PersonName: ...

class DataElement:
    descripWidth: int
    maxBytesToDisplay: int
    showVR: bool
    is_raw: bool
    tag: Incomplete
    VR: Incomplete
    validation_mode: Incomplete
    file_tell: Incomplete
    is_undefined_length: Incomplete
    private_creator: Incomplete
    def __init__(self, tag: int | str | tuple[int, int], VR: str, value: Any, file_value_tell: int | None = None, is_undefined_length: bool = False, already_converted: bool = False, validation_mode: int | None = None) -> None: ...
    def validate(self, value: Any) -> None: ...
    @classmethod
    def from_json(cls, dataset_class: type['Dataset'], tag: str, vr: str, value: Any, value_key: str | None, bulk_data_uri_handler: Callable[[str, str, str], BulkDataType] | Callable[[str], BulkDataType] | None = None) -> DataElement: ...
    def to_json_dict(self, bulk_data_element_handler: Callable[[DataElement], str] | None, bulk_data_threshold: int) -> dict[str, Any]: ...
    def to_json(self, bulk_data_threshold: int = 1024, bulk_data_element_handler: Callable[[DataElement], str] | None = None, dump_handler: Callable[[dict[str, Any]], str] | None = None) -> str: ...
    @property
    def value(self) -> Any: ...
    @value.setter
    def value(self, val: Any) -> None: ...
    @property
    def VM(self) -> int: ...
    @property
    def is_buffered(self) -> bool: ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def empty_value(self) -> bytes | list[str] | None | str | PersonName: ...
    def clear(self) -> None: ...
    def __deepcopy__(self, memo: dict[int, Any]) -> DataElement: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    @property
    def repval(self) -> str: ...
    def __getitem__(self, key: int) -> Any: ...
    @property
    def name(self) -> str: ...
    @property
    def is_private(self) -> bool: ...
    @property
    def is_retired(self) -> bool: ...
    @property
    def keyword(self) -> str: ...

class RawDataElement(NamedTuple):
    tag: BaseTag
    VR: str | None
    length: int
    value: bytes | None
    value_tell: int
    is_implicit_VR: bool
    is_little_endian: bool
    is_raw: bool = ...
    is_buffered: bool = ...

def convert_raw_data_element(raw: RawDataElement, *, encoding: str | MutableSequence[str] | None = None, ds: Dataset | None = None) -> DataElement: ...
def __getattr__(name: str) -> Any: ...



from _typeshed import Incomplete
from collections.abc import MutableSequence, Sequence
from pydicom import config as config
from pydicom.dataelem import DataElement as DataElement
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.valuerep import CUSTOMIZABLE_CHARSET_VR as CUSTOMIZABLE_CHARSET_VR, PersonName as PersonName, TEXT_VR_DELIMS as TEXT_VR_DELIMS, VR as VR

default_encoding: str
python_encoding: Incomplete
STAND_ALONE_ENCODINGS: Incomplete
ESC: bytes
CODES_TO_ENCODINGS: Incomplete
ENCODINGS_TO_CODES: Incomplete
handled_encodings: Incomplete
need_tail_escape_sequence_encodings: Incomplete
custom_encoders: Incomplete

def decode_bytes(value: bytes, encodings: Sequence[str], delimiters: set[int]) -> str: ...
decode_string = decode_bytes

def encode_string(value: str, encodings: Sequence[str]) -> bytes: ...
def convert_encodings(encodings: None | str | MutableSequence[str]) -> list[str]: ...
def decode_element(elem: DataElement, dicom_character_set: str | list[str] | None) -> None: ...



import os
from pydicom.dataset import Dataset as Dataset
from pydicom.valuerep import VR as VR
from typing import BinaryIO

def print_character(ordchr: int) -> str: ...
def filedump(filename: str | bytes | os.PathLike, start_address: int = 0, stop_address: int | None = None) -> str: ...
def datadump(data: bytes, start_address: int = 0, stop_address: int | None = None) -> str: ...
def hexdump(f: BinaryIO, start_address: int = 0, stop_address: int | None = None, show_address: bool = True) -> str: ...
def pretty_print(ds: Dataset, indent_level: int = 0, indent_chars: str = '  ') -> None: ...



import pathlib
import tqdm
from . import retry as retry
from _typeshed import Incomplete
from pydicom.misc import warn_and_log as warn_and_log

HAVE_REQUESTS: bool

class DownloadProgressBar(tqdm.tqdm):
    total: Incomplete
    def update_to(self, b: int = 1, bsize: int = 1, tsize: int | None = None) -> None: ...

USE_PROGRESS_BAR: bool
HERE: Incomplete

def calculate_file_hash(fpath: pathlib.Path) -> str: ...
def get_config_dir() -> pathlib.Path: ...
def download_with_progress(url: str, fpath: pathlib.Path) -> None: ...
def get_data_dir() -> pathlib.Path: ...
def get_url_map() -> dict[str, str]: ...
def get_url(filename: str) -> str: ...
def data_path_with_download(filename: str, check_hash: bool = True, redownload_on_hash_mismatch: bool = True, url: str | None = None, quiet: bool = True) -> pathlib.Path: ...

class NoHashFound(KeyError): ...

def get_cached_filehash(filename: str) -> str: ...
def data_file_hash_check(filename: str) -> bool: ...



import numpy as np
from _typeshed import Incomplete
from collections.abc import Iterable, Iterator
from pydicom import config as config
from pydicom.dataset import Dataset as Dataset
from pydicom.pixels.common import Buffer as Buffer, CoderBase as CoderBase, RunnerBase as RunnerBase, RunnerOptions as RunnerOptions
from pydicom.uid import JPEG2000 as JPEG2000, JPEG2000Lossless as JPEG2000Lossless, JPEGBaseline8Bit as JPEGBaseline8Bit, JPEGExtended12Bit as JPEGExtended12Bit, JPEGLSLossless as JPEGLSLossless, JPEGLSNearLossless as JPEGLSNearLossless, JPEGLSTransferSyntaxes as JPEGLSTransferSyntaxes, JPEGLossless as JPEGLossless, JPEGLosslessSV1 as JPEGLosslessSV1, RLELossless as RLELossless, UID as UID
from typing import Any

LOGGER: Incomplete
EncodeFunction: Incomplete

class EncodeOptions(RunnerOptions, total=False):
    byteorder: str
    jls_error: int
    j2k_cr: list[float]
    j2k_psnr: list[float]

class EncodeRunner(RunnerBase):
    def __init__(self, tsyntax: UID) -> None: ...
    def encode(self, index: int | None) -> bytes: ...
    def get_frame(self, index: int | None) -> bytes: ...
    def set_encoders(self, encoders: dict[str, EncodeFunction]) -> None: ...
    def set_source(self, src: np.ndarray | Dataset | Buffer) -> None: ...
    @property
    def src(self) -> Buffer | np.ndarray: ...
    def validate(self) -> None: ...

class Encoder(CoderBase):
    def __init__(self, uid: UID) -> None: ...
    def encode(self, src: bytes | np.ndarray | Dataset, *, index: int | None = None, validate: bool = True, encoding_plugin: str = '', **kwargs: Any) -> bytes: ...
    def iter_encode(self, src: bytes | np.ndarray | Dataset, *, validate: bool = True, encoding_plugin: str = '', **kwargs: Any) -> Iterator[bytes]: ...
ProfileType = tuple[str, int, Iterable[int], Iterable[int], Iterable[int]]
ENCODING_PROFILES: dict[UID, list[ProfileType]]
RLELosslessEncoder: Incomplete
JPEGLSLosslessEncoder: Incomplete
JPEGLSNearLosslessEncoder: Incomplete
JPEG2000LosslessEncoder: Incomplete
JPEG2000Encoder: Incomplete

def get_encoder(uid: str) -> Encoder: ...



import numpy
import pydicom.uid
import pydicom.valuerep
from _typeshed import Incomplete
from collections.abc import Callable, MutableSequence
from pydicom import config as config
from pydicom.charset import decode_bytes as decode_bytes, default_encoding as default_encoding
from pydicom.config import have_numpy as have_numpy, logger as logger
from pydicom.dataelem import RawDataElement as RawDataElement, empty_value_for_VR as empty_value_for_VR
from pydicom.errors import BytesLengthException as BytesLengthException
from pydicom.filereader import read_sequence as read_sequence
from pydicom.multival import MultiValue as MultiValue
from pydicom.sequence import Sequence as Sequence
from pydicom.tag import BaseTag as BaseTag, Tag as Tag, TupleTag as TupleTag
from pydicom.valuerep import CUSTOMIZABLE_CHARSET_VR as CUSTOMIZABLE_CHARSET_VR, DA as DA, DT as DT, IS as IS, PersonName as PersonName, TEXT_VR_DELIMS as TEXT_VR_DELIMS, TM as TM, validate_value as validate_value
from typing import Any

def multi_string(val: str, valtype: Callable[[str], _T] | None = None) -> _T | MutableSequence[_T]: ...
def convert_tag(byte_string: bytes, is_little_endian: bool, offset: int = 0) -> BaseTag: ...
def convert_AE_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str | MutableSequence[str]: ...
def convert_ATvalue(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> BaseTag | MutableSequence[BaseTag]: ...
def convert_DA_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str | DA | MutableSequence[str] | MutableSequence[DA]: ...
def convert_DS_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> pydicom.valuerep.DSclass | MutableSequence[pydicom.valuerep.DSclass] | numpy.float64 | numpy.ndarray: ...
def convert_DT_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str | DT | MutableSequence[str] | MutableSequence[DT]: ...
def convert_IS_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> IS | MutableSequence[IS] | numpy.int64 | numpy.ndarray: ...
def convert_numbers(byte_string: bytes, is_little_endian: bool, struct_format: str) -> str | int | float | MutableSequence[int] | MutableSequence[float]: ...
def convert_OBvalue(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> bytes: ...
def convert_OWvalue(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> bytes: ...
def convert_OVvalue(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> bytes: ...
def convert_PN(byte_string: bytes, encodings: list[str] | None = None) -> PersonName | MutableSequence[PersonName]: ...
def convert_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str | MutableSequence[str]: ...
def convert_text(byte_string: bytes, encodings: list[str] | None = None, vr: str | None = None) -> str | MutableSequence[str]: ...
def convert_single_string(byte_string: bytes, encodings: list[str] | None = None, vr: str | None = None) -> str: ...
def convert_SQ(byte_string: bytes, is_implicit_VR: bool, is_little_endian: bool, encoding: list[str] | None = None, offset: int = 0) -> Sequence: ...
def convert_TM_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str | TM | MutableSequence[str] | MutableSequence[TM]: ...
def convert_UI(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> pydicom.uid.UID | MutableSequence[pydicom.uid.UID]: ...
def convert_UN(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> bytes: ...
def convert_UR_string(byte_string: bytes, is_little_endian: bool, struct_format: str | None = None) -> str: ...
def convert_value(VR: str, raw_data_element: RawDataElement, encodings: str | MutableSequence[str] | None = None) -> Any | MutableSequence[Any]: ...

convert_retry_VR_order: Incomplete
converters: Incomplete



import os
from _typeshed import Incomplete
from enum import IntEnum
from pydicom import Dataset as Dataset
from pydicom.data.download import calculate_file_hash as calculate_file_hash, data_path_with_download as data_path_with_download, get_cached_filehash as get_cached_filehash, get_data_dir as get_data_dir, get_url_map as get_url_map
from pydicom.misc import warn_and_log as warn_and_log

DATA_ROOT: Incomplete

class DataTypes(IntEnum):
    DATASET = 0
    CHARSET = 1
    PALETTE = 2
    DICOMDIR = 3
    JPEG = 4

def get_external_sources() -> dict: ...
def external_data_sources() -> dict: ...
def online_test_file_dummy_paths() -> dict[str, str]: ...
def fetch_data_files() -> None: ...
def get_files(base: str | os.PathLike, pattern: str = '**/*', dtype: int = ...) -> list[str]: ...
def get_palette_files(pattern: str = '**/*') -> list[str]: ...
def get_testdata_file(name: str, read: bool = False, download: bool = True) -> str | Dataset | None: ...
def get_testdata_files(pattern: str = '**/*') -> list[str]: ...
def get_charset_files(pattern: str = '**/*') -> list[str]: ...



from collections.abc import MutableSequence
import datetime
from decimal import Decimal

from pydicom.dataset import Dataset
from pydicom.tag import BaseTag
from pydicom.uid import UID
from pydicom.valuerep import PersonName, DA, DSfloat, DSdecimal, DT, IS, ISfloat, TM

BytesSequence = MutableSequence[bytes]
StrSequence = MutableSequence[str]
IntSequence = MutableSequence[int]
FloatSequence = MutableSequence[float]

BytesType = None | bytes
BytesListType = None | BytesSequence
DateType = None | str | datetime.date | DA
DateListType = None | StrSequence | MutableSequence[datetime.date] | MutableSequence[DA]
DateTimeType = None | str | datetime.datetime | DT
DateTimeListType = (
    None | StrSequence | MutableSequence[datetime.datetime] | MutableSequence[DT]
)
DecimalStringType = None | str | int | float | Decimal | DSfloat | DSdecimal
DecimalStringListType = (
    None
    | StrSequence
    | IntSequence
    | FloatSequence
    | MutableSequence[Decimal]
    | MutableSequence[DSfloat]
    | MutableSequence[DSdecimal]
)
FloatType = None | float
FloatListType = None | FloatSequence
IntType = None | int
IntListType = None | IntSequence
IntegerStringType = None | str | int | IS | ISfloat
IntegerStringListType = (
    None | StrSequence | IntSequence | MutableSequence[IS] | MutableSequence[ISfloat]
)
PersonNameType = None | str | PersonName
PersonNameListType = None | StrSequence | MutableSequence[PersonName]
StringType = None | str
StringListType = None | StrSequence
TagType = None | int | str | tuple[int, int] | BaseTag
TagListType = (
    None
    | IntSequence
    | StrSequence
    | MutableSequence[tuple[int, int]]
    | MutableSequence[BaseTag]
)
TimeType = None | str | datetime.time | TM
TimeListType = None | StrSequence | MutableSequence[datetime.time] | MutableSequence[TM]
UIDType = None | str | UID
UIDListType = None | StrSequence | MutableSequence[UID]

# Standard VRs
# _1: VM is 1, _1N: VM is 1-n, _N: VM is 2+-n
AE_1_Type = StringType
AE_1N_Type = StringType | StringListType
AE_N_Type = StringListType

AS_1_Type = StringType
AS_1N_Type = StringType | StringListType
AS_N_Type = StringListType

AT_1_Type = TagType
AT_1N_Type = TagType | TagListType
AT_N_Type = TagListType

CS_1_Type = StringType
CS_1N_Type = StringType | StringListType
CS_N_Type = StringListType

DA_1_Type = DateType
DA_1N_Type = DateType | DateListType
DA_N_Type = DateListType

DS_1_Type = DecimalStringType
DS_1N_Type = DecimalStringType | DecimalStringListType
DS_N_Type = DecimalStringListType

DT_1_Type = DateTimeType
DT_1N_Type = DateTimeType | DateTimeListType
DT_N_Type = DateTimeListType

FD_1_Type = FloatType
FD_1N_Type = FloatType | FloatListType
FD_N_Type = FloatListType

FL_1_Type = FloatType
FL_1N_Type = FloatType | FloatListType
FL_N_Type = FloatListType

IS_1_Type = IntegerStringType
IS_1N_Type = IntegerStringType | IntegerStringListType
IS_N_Type = IntegerStringListType

LO_1_Type = StringType
LO_1N_Type = StringType | StringListType
LO_N_Type = StringListType

LT_1_Type = StringType
LT_1N_Type = StringType | StringListType
LT_N_Type = StringListType

OB_1_Type = BytesType
OB_1N_Type = BytesType | BytesListType
OB_N_Type = BytesListType

OD_1_Type = BytesType
OD_1N_Type = BytesType | BytesListType
OD_N_Type = BytesListType

OF_1_Type = BytesType
OF_1N_Type = BytesType | BytesListType
OF_N_Type = BytesListType

OL_1_Type = BytesType
OL_1N_Type = BytesType | BytesListType
OL_N_Type = BytesListType

OW_1_Type = BytesType
OW_1N_Type = BytesType | BytesListType
OW_N_Type = BytesListType

OV_1_Type = BytesType
OV_1N_Type = BytesType | BytesListType
OV_N_Type = BytesListType

PN_1_Type = PersonNameType
PN_1N_Type = PersonNameType | PersonNameListType
PN_N_Type = PersonNameListType

SH_1_Type = StringType
SH_1N_Type = StringType | StringListType
SH_N_Type = StringListType

SL_1_Type = IntType
SL_1N_Type = IntType | IntListType
SL_N_Type = IntListType

SQType = None | MutableSequence[Dataset]

SS_1_Type = IntType
SS_1N_Type = IntType | IntListType
SS_N_Type = IntListType

ST_1_Type = StringType
ST_1N_Type = StringType | StringListType
ST_N_Type = StringListType

SV_1_Type = IntType
SV_1N_Type = IntType | IntListType
SV_N_Type = IntListType

TM_1_Type = TimeType
TM_1N_Type = TimeType | TimeListType
TM_N_Type = TimeListType

UC_1_Type = StringType
UC_1N_Type = StringType | StringListType
UC_N_Type = StringListType

UI_1_Type = UIDType
UI_1N_Type = UIDType | UIDListType
UI_N_Type = UIDListType

UL_1_Type = IntType
UL_1N_Type = IntType | IntListType
UL_N_Type = IntListType

UN_1_Type = BytesType
UN_1N_Type = BytesType | BytesListType
UN_N_Type = BytesListType

UR_1_Type = StringType
UR_1N_Type = StringType | StringListType
UR_N_Type = StringListType

US_1_Type = IntType
US_1N_Type = IntType | IntListType
US_N_Type = IntListType

UT_1_Type = StringType
UT_1N_Type = StringType | StringListType
UT_N_Type = StringListType

UV_1_Type = IntType
UV_1N_Type = IntType | IntListType
UV_N_Type = IntListType

# Ambiguous VRs
# US or SS or OW: 1, 1-n
US_SS_OW_Type = IntType | BytesType | IntListType | BytesListType
# US or OW: 1, 1-n
US_OW_Type = IntType | BytesType | IntListType | BytesListType
# US or SS: 1, 3, 4
US_SS_Type = IntType | IntListType
# OB or OW: 1
OB_OW_Type = BytesType



import numpy as np
from _typeshed import Incomplete
from collections.abc import Iterable, Iterator
from pydicom import config as config
from pydicom.dataset import Dataset as Dataset
from pydicom.encaps import generate_frames as generate_frames, get_frame as get_frame
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.pixels.common import Buffer as Buffer, CoderBase as CoderBase, RunnerBase as RunnerBase, RunnerOptions as RunnerOptions
from pydicom.pixels.processing import convert_color_space as convert_color_space
from pydicom.pixels.utils import get_j2k_parameters as get_j2k_parameters
from pydicom.uid import DeflatedExplicitVRLittleEndian as DeflatedExplicitVRLittleEndian, ExplicitVRBigEndian as ExplicitVRBigEndian, ExplicitVRLittleEndian as ExplicitVRLittleEndian, HTJ2K as HTJ2K, HTJ2KLossless as HTJ2KLossless, HTJ2KLosslessRPCL as HTJ2KLosslessRPCL, ImplicitVRLittleEndian as ImplicitVRLittleEndian, JPEG2000 as JPEG2000, JPEG2000Lossless as JPEG2000Lossless, JPEG2000TransferSyntaxes as JPEG2000TransferSyntaxes, JPEGBaseline8Bit as JPEGBaseline8Bit, JPEGExtended12Bit as JPEGExtended12Bit, JPEGLSLossless as JPEGLSLossless, JPEGLSNearLossless as JPEGLSNearLossless, JPEGLSTransferSyntaxes as JPEGLSTransferSyntaxes, JPEGLossless as JPEGLossless, JPEGLosslessSV1 as JPEGLosslessSV1, JPEGTransferSyntaxes as JPEGTransferSyntaxes, RLELossless as RLELossless, UID as UID
from typing import Any, BinaryIO

HAVE_NP: bool
LOGGER: Incomplete
DecodeFunction: Incomplete
ProcessingFunction: Incomplete

class DecodeOptions(RunnerOptions, total=False):
    pixel_vr: str
    allow_excess_frames: bool
    correct_unused_bits: bool
    view_only: bool
    be_swap_ow: bool
    rle_segment_order: str
    byteorder: str
    apply_jls_sign_correction: bool
    apply_j2k_sign_correction: bool
    as_rgb: bool
    force_rgb: bool
    force_ybr: bool

PROCESSORS: list[ProcessingFunction]

class DecodeRunner(RunnerBase):
    def __init__(self, tsyntax: UID) -> None: ...
    def decode(self, index: int) -> bytes | bytearray: ...
    def get_data(self, src: Buffer | BinaryIO, offset: int, length: int) -> bytes: ...
    def iter_decode(self) -> Iterator[bytes | bytearray]: ...
    @property
    def pixel_dtype(self) -> np.dtype: ...
    def pixel_properties(self, as_frame: bool = False) -> dict[str, str | int]: ...
    def process(self, arr: np.ndarray) -> tuple['np.ndarray', dict[str, str | int]]: ...
    def reshape(self, arr: np.ndarray, as_frame: bool = False) -> np.ndarray: ...
    def set_decoders(self, decoders: dict[str, DecodeFunction]) -> None: ...
    def set_source(self, src: Buffer | Dataset | BinaryIO) -> None: ...
    @property
    def src(self) -> Buffer | BinaryIO: ...
    def validate(self) -> None: ...

class Decoder(CoderBase):
    def __init__(self, uid: UID) -> None: ...
    def as_array(self, src: Dataset | Buffer | BinaryIO, *, index: int | None = None, validate: bool = True, raw: bool = False, decoding_plugin: str = '', **kwargs: DecodeOptions) -> tuple['np.ndarray', dict[str, str | int]]: ...
    def as_buffer(self, src: Dataset | Buffer | BinaryIO, *, index: int | None = None, validate: bool = True, decoding_plugin: str = '', **kwargs: Any) -> tuple[Buffer, dict[str, str | int]]: ...
    def iter_array(self, src: Dataset | Buffer | BinaryIO, *, indices: Iterable[int] | None = None, raw: bool = False, validate: bool = True, decoding_plugin: str = '', **kwargs: Any) -> Iterator[tuple['np.ndarray', dict[str, str | int]]]: ...
    def iter_buffer(self, src: Dataset | Buffer | BinaryIO, *, indices: Iterable[int] | None = None, validate: bool = True, decoding_plugin: str = '', **kwargs: Any) -> Iterator[tuple[Buffer, dict[str, str | int]]]: ...

ImplicitVRLittleEndianDecoder: Incomplete
ExplicitVRLittleEndianDecoder: Incomplete
ExplicitVRBigEndianDecoder: Incomplete
DeflatedExplicitVRLittleEndianDecoder: Incomplete
JPEGBaseline8BitDecoder: Incomplete
JPEGExtended12BitDecoder: Incomplete
JPEGLosslessDecoder: Incomplete
JPEGLosslessSV1Decoder: Incomplete
JPEGLSLosslessDecoder: Incomplete
JPEGLSNearLosslessDecoder: Incomplete
JPEG2000LosslessDecoder: Incomplete
JPEG2000Decoder: Incomplete
HTJ2KLosslessDecoder: Incomplete
HTJ2KLosslessRPCLDecoder: Incomplete
HTJ2KDecoder: Incomplete
RLELosslessDecoder: Incomplete

def get_decoder(uid: str) -> Decoder: ...



import numpy as np
from _typeshed import Incomplete
from pydicom.dataelem import DataElement as DataElement
from pydicom.dataset import Dataset as Dataset
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.pixels.utils import unpack_bits as unpack_bits
from typing import Any

HAVE_NP: bool
HANDLER_NAME: str
DEPENDENCIES: Incomplete

def is_available() -> bool: ...
def get_expected_length(elem: dict[str, Any], unit: str = 'bytes') -> int: ...
def reshape_overlay_array(elem: dict[str, Any], arr: np.ndarray) -> np.ndarray: ...
def get_overlay_array(ds: Dataset, group: int) -> np.ndarray: ...



import os
from _typeshed import Incomplete
from collections.abc import Callable as Callable, Iterator
from pydicom.datadict import dictionary_VR as dictionary_VR
from pydicom.misc import size_in_bytes as size_in_bytes
from pydicom.tag import ItemTag as ItemTag, TupleTag as TupleTag
from pydicom.uid import UID as UID
from pydicom.valuerep import EXPLICIT_VR_LENGTH_32 as EXPLICIT_VR_LENGTH_32
from types import TracebackType
from typing import BinaryIO

extra_length_VRs_b: Incomplete
ExplicitVRLittleEndian: bytes
ImplicitVRLittleEndian: bytes
DeflatedExplicitVRLittleEndian: bytes
ExplicitVRBigEndian: bytes

class dicomfile:
    fobj: Incomplete
    preamble: Incomplete
    def __init__(self, filename: str | bytes | os.PathLike) -> None: ...
    def __enter__(self) -> dicomfile: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None) -> bool | None: ...
    def __iter__(self) -> Iterator[_ElementType]: ...

def data_element_generator(fp: BinaryIO, is_implicit_VR: bool, is_little_endian: bool, stop_when: Callable[[int, int], bool] | None = None, defer_size: str | int | float | None = None) -> Iterator[_ElementType]: ...



import os
from _typeshed import Incomplete
from collections.abc import Callable as Callable, Iterable, Iterator
from pathlib import Path
from pydicom.charset import default_encoding as default_encoding
from pydicom.datadict import dictionary_description as dictionary_description, tag_for_keyword as tag_for_keyword
from pydicom.dataelem import DataElement as DataElement
from pydicom.dataset import Dataset as Dataset, FileDataset as FileDataset, FileMetaDataset as FileMetaDataset
from pydicom.filebase import DicomBytesIO as DicomBytesIO, DicomFileLike as DicomFileLike
from pydicom.filereader import dcmread as dcmread
from pydicom.filewriter import write_data_element as write_data_element, write_dataset as write_dataset, write_file_meta_info as write_file_meta_info
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.tag import BaseTag as BaseTag, Tag as Tag
from pydicom.uid import ExplicitVRLittleEndian as ExplicitVRLittleEndian, ImplicitVRLittleEndian as ImplicitVRLittleEndian, MediaStorageDirectoryStorage as MediaStorageDirectoryStorage, UID as UID, generate_uid as generate_uid
from typing import Any

def generate_filename(prefix: str = '', start: int = 0, alphanumeric: bool = False) -> Iterator[str]: ...
def is_conformant_file_id(path: Path) -> bool: ...

class RecordNode(Iterable['RecordNode']):
    children: Incomplete
    instance: Incomplete
    def __init__(self, record: Dataset | None = None) -> None: ...
    def add(self, leaf: RecordNode) -> None: ...
    @property
    def ancestors(self) -> list['RecordNode']: ...
    @property
    def component(self) -> str: ...
    def __contains__(self, key: str | RecordNode) -> bool: ...
    def __delitem__(self, key: str | RecordNode) -> None: ...
    @property
    def depth(self) -> int: ...
    @property
    def file_set(self) -> FileSet: ...
    def __getitem__(self, key: str | RecordNode) -> RecordNode: ...
    @property
    def has_instance(self) -> bool: ...
    @property
    def index(self) -> int: ...
    @property
    def is_root(self) -> bool: ...
    def __iter__(self) -> Iterator['RecordNode']: ...
    @property
    def key(self) -> str: ...
    @property
    def next(self) -> RecordNode | None: ...
    @property
    def parent(self) -> RecordNode: ...
    @parent.setter
    def parent(self, node: RecordNode) -> None: ...
    def prettify(self, indent_char: str = '  ') -> list[str]: ...
    @property
    def previous(self) -> RecordNode | None: ...
    @property
    def record_type(self) -> str: ...
    def remove(self, node: RecordNode) -> None: ...
    def reverse(self) -> Iterable['RecordNode']: ...
    @property
    def root(self) -> RecordNode: ...

class RootNode(RecordNode):
    def __init__(self, fs: FileSet) -> None: ...
    @property
    def file_set(self) -> FileSet: ...
    @property
    def is_root(self) -> bool: ...

class FileInstance:
    node: Incomplete
    def __init__(self, node: RecordNode) -> None: ...
    def __contains__(self, name: str | int) -> bool: ...
    @property
    def FileID(self) -> str: ...
    @property
    def file_set(self) -> FileSet: ...
    @property
    def for_addition(self) -> bool: ...
    @property
    def for_moving(self) -> bool: ...
    @property
    def for_removal(self) -> bool: ...
    def __getattribute__(self, name: str) -> Any: ...
    def __getitem__(self, key: str | int) -> DataElement: ...
    @property
    def is_private(self) -> bool: ...
    @property
    def is_staged(self) -> bool: ...
    def load(self) -> Dataset: ...
    @property
    def path(self) -> str: ...
    @property
    def SOPClassUID(self) -> UID: ...
    @property
    def SOPInstanceUID(self) -> UID: ...
    @property
    def TransferSyntaxUID(self) -> UID: ...
DSPathType = Dataset | str | os.PathLike

class FileSet:
    def __init__(self, ds: DSPathType | None = None) -> None: ...
    def add(self, ds_or_path: DSPathType) -> FileInstance: ...
    def add_custom(self, ds_or_path: DSPathType, leaf: RecordNode) -> FileInstance: ...
    def clear(self) -> None: ...
    def copy(self, path: str | os.PathLike, force_implicit: bool = False) -> FileSet: ...
    @property
    def descriptor_character_set(self) -> str | None: ...
    @descriptor_character_set.setter
    def descriptor_character_set(self, val: str | None) -> None: ...
    @property
    def descriptor_file_id(self) -> str | None: ...
    @descriptor_file_id.setter
    def descriptor_file_id(self, val: str | None) -> None: ...
    def find(self, load: bool = False, **kwargs: Any) -> list[FileInstance]: ...
    def find_values(self, elements: str | int | list[str | int], instances: list[FileInstance] | None = None, load: bool = False) -> list[Any] | dict[str | int, list[Any]]: ...
    @property
    def ID(self) -> str | None: ...
    @ID.setter
    def ID(self, val: str | None) -> None: ...
    @property
    def is_staged(self) -> bool: ...
    def __iter__(self) -> Iterator[FileInstance]: ...
    def __len__(self) -> int: ...
    def load(self, ds_or_path: DSPathType, include_orphans: bool = True, raise_orphans: bool = False) -> None: ...
    @property
    def path(self) -> str | None: ...
    def remove(self, instance: FileInstance | list[FileInstance]) -> None: ...
    @property
    def UID(self) -> UID: ...
    @UID.setter
    def UID(self, uid: UID) -> None: ...
    def write(self, path: str | os.PathLike | None = None, use_existing: bool = False, force_implicit: bool = False) -> None: ...

DIRECTORY_RECORDERS: Incomplete



import logging
from collections.abc import Callable as Callable
from typing import Any

def retry(exc: type[Exception] | tuple[type[Exception], ...], exc_msg: str | None = None, tries: int = 4, delay: int = 3, backoff: int = 2, logger: logging.Logger | None = None) -> Callable[[Callable], Any]: ...



import numpy as np
from _typeshed import Incomplete
from collections.abc import Generator
from pydicom.dataset import Dataset as Dataset

HAVE_NP: bool
HANDLER_NAME: str
DEPENDENCIES: Incomplete
WAVEFORM_DTYPES: Incomplete

def is_available() -> bool: ...
def generate_multiplex(ds: Dataset, as_raw: bool = True) -> Generator['np.ndarray', None, None]: ...
def multiplex_array(ds: Dataset, index: int, as_raw: bool = True) -> np.ndarray: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from io import BufferedIOBase
from pydicom import config as config
from pydicom.filebase import DicomBytesIO as DicomBytesIO, DicomIO as DicomIO, ReadableBuffer as ReadableBuffer
from pydicom.fileutil import buffer_length as buffer_length, reset_buffer_position as reset_buffer_position
from pydicom.misc import warn_and_log as warn_and_log
from pydicom.tag import ItemTag as ItemTag, SequenceDelimiterTag as SequenceDelimiterTag, Tag as Tag
from typing import Any

def parse_basic_offsets(buffer: bytes | bytearray | ReadableBuffer, *, endianness: str = '<') -> list[int]: ...
def parse_fragments(buffer: bytes | bytearray | ReadableBuffer, *, endianness: str = '<') -> tuple[int, list[int]]: ...
def generate_fragments(buffer: bytes | bytearray | ReadableBuffer, *, endianness: str = '<') -> Iterator[bytes]: ...
def generate_fragmented_frames(buffer: bytes | bytearray | ReadableBuffer, *, number_of_frames: int | None = None, extended_offsets: tuple[list[int], list[int]] | tuple[bytes, bytes] | None = None, endianness: str = '<') -> Iterator[tuple[bytes, ...]]: ...
def generate_frames(buffer: bytes | ReadableBuffer, *, number_of_frames: int | None = None, extended_offsets: tuple[list[int], list[int]] | tuple[bytes, bytes] | None = None, endianness: str = '<') -> Iterator[bytes]: ...
def get_frame(buffer: bytes | bytearray | ReadableBuffer, index: int, *, extended_offsets: tuple[list[int], list[int]] | tuple[bytes, bytes] | None = None, number_of_frames: int | None = None, endianness: str = '<') -> bytes: ...

class _BufferedItem:
    buffer: Incomplete
    length: Incomplete
    def __init__(self, buffer: BufferedIOBase) -> None: ...
    def read(self, start: int, size: int) -> bytes: ...

class EncapsulatedBuffer(BufferedIOBase):
    def __init__(self, buffers: list[BufferedIOBase], use_bot: bool = False) -> None: ...
    @property
    def basic_offset_table(self) -> bytes: ...
    @property
    def closed(self) -> bool: ...
    @property
    def extended_lengths(self) -> bytes: ...
    @property
    def extended_offsets(self) -> bytes: ...
    @property
    def encapsulated_length(self) -> int: ...
    @property
    def lengths(self) -> list[int]: ...
    @property
    def offsets(self) -> list[int]: ...
    def read(self, size: int | None = 8192, /) -> bytes: ...
    def readable(self) -> bool: ...
    def seek(self, offset: int, whence: int = ..., /) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...

def fragment_frame(frame: bytes, nr_fragments: int = 1) -> Iterator[bytes]: ...
def itemize_fragment(fragment: bytes) -> bytes: ...
def itemize_frame(frame: bytes, nr_fragments: int = 1) -> Iterator[bytes]: ...
def encapsulate(frames: list[bytes], fragments_per_frame: int = 1, has_bot: bool = True) -> bytes: ...
def encapsulate_buffer(buffers: list[BufferedIOBase], has_bot: bool = True) -> EncapsulatedBuffer: ...
def encapsulate_extended(frames: list[bytes]) -> tuple[bytes, bytes, bytes]: ...
def encapsulate_extended_buffer(buffers: list[BufferedIOBase]) -> tuple[EncapsulatedBuffer, bytes, bytes]: ...
def __getattr__(name: str) -> Any: ...



from _typeshed import Incomplete
from collections.abc import Callable as Callable, Iterator, MutableSequence
from pydicom import config as config
from pydicom.charset import convert_encodings as convert_encodings, default_encoding as default_encoding
from pydicom.config import logger as logger
from pydicom.dataelem import DataElement as DataElement, RawDataElement as RawDataElement, convert_raw_data_element as convert_raw_data_element, empty_value_for_VR as empty_value_for_VR
from pydicom.dataset import Dataset as Dataset, FileDataset as FileDataset, FileMetaDataset as FileMetaDataset
from pydicom.errors import InvalidDicomError as InvalidDicomError
from pydicom.filebase import DicomBytesIO as DicomBytesIO, ReadableBuffer as ReadableBuffer
from pydicom.fileutil import PathType as PathType, path_from_pathlike as path_from_pathlike, read_undefined_length_value as read_undefined_length_value
from pydicom.misc import size_in_bytes as size_in_bytes, warn_and_log as warn_and_log
from pydicom.sequence import Sequence as Sequence
from pydicom.tag import BaseTag as BaseTag, ItemTag as ItemTag, SequenceDelimiterTag as SequenceDelimiterTag, Tag as Tag, TagListType as TagListType
from pydicom.util.hexutil import bytes2hex as bytes2hex
from pydicom.valuerep import EXPLICIT_VR_LENGTH_32 as EXPLICIT_VR_LENGTH_32
from typing import Any, BinaryIO

ENCODED_VR: Incomplete

def data_element_generator(fp: BinaryIO, is_implicit_VR: bool, is_little_endian: bool, stop_when: Callable[[BaseTag, str | None, int], bool] | None = None, defer_size: int | str | float | None = None, encoding: str | MutableSequence[str] = ..., specific_tags: list[BaseTag | int] | None = None) -> Iterator[RawDataElement | DataElement]: ...
def read_dataset(fp: BinaryIO, is_implicit_VR: bool, is_little_endian: bool, bytelength: int | None = None, stop_when: Callable[[BaseTag, str | None, int], bool] | None = None, defer_size: str | int | float | None = None, parent_encoding: str | MutableSequence[str] = ..., specific_tags: list[BaseTag | int] | None = None, at_top_level: bool = True) -> Dataset: ...
def read_sequence(fp: BinaryIO, is_implicit_VR: bool, is_little_endian: bool, bytelength: int, encoding: str | MutableSequence[str], offset: int = 0) -> Sequence: ...
def read_sequence_item(fp: BinaryIO, is_implicit_VR: bool, is_little_endian: bool, encoding: str | MutableSequence[str], offset: int = 0) -> Dataset | None: ...
def read_file_meta_info(filename: PathType) -> FileMetaDataset: ...
def read_preamble(fp: BinaryIO, force: bool) -> bytes | None: ...
def read_partial(fileobj: BinaryIO, stop_when: Callable[[BaseTag, str | None, int], bool] | None = None, defer_size: int | str | float | None = None, force: bool = False, specific_tags: list[BaseTag | int] | None = None) -> FileDataset: ...
def dcmread(fp: PathType | BinaryIO | ReadableBuffer, defer_size: str | int | float | None = None, stop_before_pixels: bool = False, force: bool = False, specific_tags: TagListType | None = None) -> FileDataset: ...
def data_element_offset_to_value(is_implicit_VR: bool, VR: str | None) -> int: ...
def read_deferred_data_element(fileobj_type: Any, filename_or_obj: PathType | BinaryIO, timestamp: float | None, raw_data_elem: RawDataElement) -> RawDataElement: ...



from .data_manager import fetch_data_files as fetch_data_files, get_charset_files as get_charset_files, get_palette_files as get_palette_files, get_testdata_file as get_testdata_file, get_testdata_files as get_testdata_files

__all__ = ['fetch_data_files', 'get_charset_files', 'get_palette_files', 'get_testdata_files', 'get_testdata_file']
